r"""
Echelon matrices over finite fields.
"""
# ****************************************************************************
#       Copyright (C) 2014 Vincent Delecroix <20100.delecroix@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

from sage.matrix.matrix0 cimport Matrix


def reduced_echelon_matrix_iterator(K, k, n, bint sparse=False, bint copy=True, bint set_immutable=False):
    r"""
    An iterator over `(k,n)` reduced echelon matrices over the finite field `K`.

    INPUT:

    - ``K`` -- a finite field

    - ``k`` -- number of rows (or the size of the subspace)

    - ``n`` -- number of columns (or the dimension of the ambient space)

    - ``sparse`` -- boolean (default: ``False``)

    - ``copy`` -- boolean (default: ``True``); if set to ``False`` then
      iterator yields the same matrix over and over (but with different
      entries).  Default is ``True`` which is safer but might be slower.

    - ``set_immutable`` -- boolean; if set to ``True`` then the output matrices
      are immutable. This option automatically turns ``copy`` into ``True``.


    .. NOTE::

        We ensure that the iteration order is so that all matrices with given
        pivot columns are generated consecutively. Furthermore, the order in
        which the pivot columns appear is lexicographic.

        It would be faster to generate the pivots columns following a Gray code.
        There would be only one pivot changing at a time, avoiding the possibly
        expensive ``m0.__copy__()``. However that would modify the generation
        order some functions depend upon.

    EXAMPLES::

        sage: from sage.matrix.echelon_matrix import reduced_echelon_matrix_iterator
        sage: it = reduced_echelon_matrix_iterator(GF(2), 2, 3)
        sage: for m in it:
        ....:     print(m)
        ....:     print(m.pivots())
        ....:     print("*******")
        [1 0 0]
        [0 1 0]
        (0, 1)
        *******
        [1 0 0]
        [0 1 1]
        (0, 1)
        *******
        [1 0 1]
        [0 1 0]
        (0, 1)
        *******
        [1 0 1]
        [0 1 1]
        (0, 1)
        *******
        [1 0 0]
        [0 0 1]
        (0, 2)
        *******
        [1 1 0]
        [0 0 1]
        (0, 2)
        *******
        [0 1 0]
        [0 0 1]
        (1, 2)
        *******

    TESTS:

    Testing cardinalities::

        sage: q = 71
        sage: F = GF(q)
        sage: len(list(reduced_echelon_matrix_iterator(F, 1, 3, copy=False))) == q**2+q+1
        True
        sage: len(list(reduced_echelon_matrix_iterator(F, 2, 3, copy=False))) == q**2+q+1
        True

    Testing options::

        sage: it = reduced_echelon_matrix_iterator(GF(4, 'z'), 2, 4, copy=False)                    # needs sage.rings.finite_rings
        sage: next(it) is next(it)                                                                  # needs sage.rings.finite_rings
        True
        sage: for a in it: pass

        sage: it = reduced_echelon_matrix_iterator(GF(4, 'z'), 2, 4, set_immutable=True)            # needs sage.rings.finite_rings
        sage: all(a.is_immutable() and a.echelon_form() == a for a in it)
        True
    """
    cdef Matrix m0, m, mm
    cdef Py_ssize_t i
    n = int(n)
    k = int(k)

    if n < k:
        raise NotImplementedError("echelon matrix with fewer rows than columns "
                                  "i.e. not full rank) are not implemented")

    from sage.matrix.matrix_space import MatrixSpace
    from itertools import combinations, product

    copy = copy or set_immutable

    m0 = MatrixSpace(K, k, n, sparse=sparse)()
    Klist = list(K)
    K1 = K.one()

    # First, we select which columns will be pivots:
    for pivots in combinations(range(n), k):
        m = m0.__copy__()
        free_positions = []
        for i in range(k):
            m[i, pivots[i]] = K1
            for j in range(pivots[i]+1, n):
                if j not in pivots:
                    free_positions.append((i, j))

        # Next, we fill in those entries that are not
        # determined by the echelon form alone:
        num_free_pos = len(free_positions)
        for v in product(Klist, repeat=num_free_pos):
            for i in range(num_free_pos):
                m[free_positions[i]] = v[i]

            if copy:
                mm = m.__copy__()
                mm.cache('pivots', pivots)
                mm.cache('rank', k)
                mm.cache('in_echelon_form', True)
                if set_immutable:
                    mm.set_immutable()
                yield mm
            else:
                yield m
            del v   # hack: Python itertools reuses the tuple if nobody else uses it
        del pivots  # hack: Python itertools reuses the tuple if nobody else uses it



r"""
Probability Distributions

This module provides three types of probability distributions:

- :class:`RealDistribution`: various real-valued probability distributions.

- :class:`SphericalDistribution`: uniformly distributed points on the
  surface of an `n-1` sphere in `n` dimensional euclidean space.

- :class:`GeneralDiscreteDistribution`: user-defined discrete distributions.

AUTHORS:

- Josh Kantor (2007-02): first version

- William Stein (2007-02): rewrite of docs, conventions, etc.

- Carlo Hamalainen (2008-08): full doctest coverage, more documentation,
  GeneralDiscreteDistribution, misc fixes.

- Kwankyu Lee (2010-05-29): F-distribution support.

REFERENCES:

    GNU gsl library, General discrete distributions
    http://www.gnu.org/software/gsl/manual/html_node/General-Discrete-Distributions.html

    GNU gsl library, Random number distributions
    http://www.gnu.org/software/gsl/manual/html_node/Random-Number-Distributions.html
"""
# ****************************************************************************
#       Copyright (C) 2004, 2005, 2006 Joshua Kantor <kantor.jm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************
from cysignals.memory cimport sig_malloc, sig_free

from sage.libs.gsl.rng cimport *
from sage.libs.gsl.random cimport *
import sage.misc.prandom as random
import sage.rings.real_double
from sage.modules.free_module_element import vector

# TODO: Add more distributions available in gsl
# available but not currently wrapped are laplace, cauchy, landau, logistic.

cdef enum:
    uniform
    gaussian
    rayleigh
    lognormal
    pareto
    t
    F
    chisquared
    exppow
    weibull
    beta
    exponential
    gamma

cdef class ProbabilityDistribution:
    r"""
    Concrete probability distributions should be derived from this
    abstract class.
    """

    def __init__(self):
        r"""
        To be implemented by a derived class::

            sage: P = sage.probability.probability_distribution.ProbabilityDistribution()
        """

        pass

    def get_random_element(self):
        r"""
        To be implemented by a derived class::

            sage: P = sage.probability.probability_distribution.ProbabilityDistribution()
            sage: P.get_random_element()
            Traceback (most recent call last):
            ...
            NotImplementedError: implement in derived class
        """

        raise NotImplementedError("implement in derived class")

    def generate_histogram_data(self, num_samples=1000, bins=50):
        r"""
        Compute a histogram of the probability distribution.

        INPUT:

        - ``num_samples`` -- (optional) number of times to sample from
          the probability distribution

        - ``bins`` -- (optional) number of bins to divide the samples
          into

        OUTPUT:

        - a tuple. The first element of the tuple is a list of length
          ``bins``, consisting of the normalised histogram of the random
          samples. The second list is the bins.

        EXAMPLES::

            sage: set_random_seed(0)
            sage: from sage.probability.probability_distribution import GeneralDiscreteDistribution
            sage: P = [0.3, 0.4, 0.3]
            sage: X = GeneralDiscreteDistribution(P)
            sage: h, b = X.generate_histogram_data(bins=10)
            sage: h  # rel tol 1e-08
            [1.445,
             0.0,
             0.0,
             0.0,
             0.0,
             2.044999999999998,
             0.0,
             0.0,
             0.0,
             1.5100000000000002]
            sage: b
            [0.0,
             0.2,
             0.4,
             0.6000000000000001,
             0.8,
             1.0,
             1.2000000000000002,
             1.4000000000000001,
             1.6,
             1.8,
             2.0]
        """
        import numpy as np
        if int(np.version.short_version[0]) > 1:
            np.set_printoptions(legacy="1.25")
        import pylab
        ell = [float(self.get_random_element()) for _ in range(num_samples)]
        S = pylab.hist(ell, bins, density=True)
        return [list(S[0]), list(S[1])]

    def generate_histogram_plot(self, name, num_samples=1000, bins=50):
        r"""
        Save the histogram from :func:`generate_histogram_data() <sage.libs.gsl.ProbabilityDistribution.generate_histogram_data>`
        to a file.

        INPUT:

        - ``name`` -- file to save the histogram plot (as a PNG)

        - ``num_samples`` -- (optional) number of times to sample from
          the probability distribution

        - ``bins`` -- (optional) number of bins to divide the samples
          into

        EXAMPLES:

        This saves the histogram plot to a temporary file::

            sage: from sage.probability.probability_distribution import GeneralDiscreteDistribution
            sage: import tempfile
            sage: P = [0.3, 0.4, 0.3]
            sage: X = GeneralDiscreteDistribution(P)
            sage: with tempfile.NamedTemporaryFile() as f:
            ....:     X.generate_histogram_plot(f.name)
        """
        import pylab
        ell = [float(self.get_random_element()) for _ in range(num_samples)]
        pylab.hist(ell, bins, density=True)
        pylab.savefig(name)


cdef class SphericalDistribution(ProbabilityDistribution):
    r"""
    This class is capable of producing random points uniformly distributed
    on the surface of an `(n-1)`-sphere in `n`-dimensional euclidean space. The
    dimension `n` is selected via the keyword ``dimension``. The random
    number generator which drives it can be selected using the keyword
    ``rng``. Valid choices are ``'default'`` which uses the Mersenne-Twister,
    ``'luxury'`` which uses RANDLXS, and ``'taus'`` which uses the tausworth
    generator. The default dimension is ``3``.

    EXAMPLES::

        sage: T = SphericalDistribution()
        sage: s = T.get_random_element()
        sage: s.norm()  # rel tol 1e-14
        1.0
        sage: len(s)
        3
        sage: T = SphericalDistribution(dimension=4, rng='luxury')
        sage: s = T.get_random_element()
        sage: s.norm()  # rel tol 1e-14
        1.0
        sage: len(s)
        4

    TESTS:

    Make sure that repeated initializations are randomly seeded
    (:issue:`9770`)::

        sage: Xs = [tuple(SphericalDistribution(2).get_random_element()) for _ in range(1000)]
        sage: len(set(Xs)) > 2^^32
        True
    """

    cdef gsl_rng *r
    cdef const gsl_rng_type *T
    cdef long int seed
    cdef Py_ssize_t dimension
    cdef double* vec

    def __init__(self, dimension=3, rng='default', seed=None):
        r"""
        EXAMPLES::

            sage: T = SphericalDistribution()
            sage: T.get_random_element().norm()  # rel tol 1e-14
            1.0

        TESTS:

        Until :issue:`15089` a value of the ``seed`` keyword
        besides ``None`` was ignored. We check here that setting
        a seed is effective. ::

            sage: T = SphericalDistribution(seed=876)
            sage: one = [T.get_random_element() for _ in range(10)]
            sage: T = SphericalDistribution(seed=876)
            sage: two = [T.get_random_element() for _ in range(10)]
            sage: T = SphericalDistribution(seed=123)
            sage: three = [T.get_random_element() for _ in range(10)]
            sage: one == two
            True
            sage: one == three
            False
        """
        gsl_rng_env_setup()
        self.set_random_number_generator(rng)
        self.r = gsl_rng_alloc(self.T)
        if seed is None:
            seed = random.randint(1, 2**31)
        self.set_seed(seed)
        self.dimension = dimension
        self.vec = <double *>sig_malloc(self.dimension*(sizeof(double)))

    def set_seed(self, seed):
        r"""
        Set the seed for the underlying random number generator.

        EXAMPLES::

            sage: T = SphericalDistribution(seed=0)
            sage: T.set_seed(100)
        """
        gsl_rng_set(self.r, seed)
        self.seed = seed

    def set_random_number_generator(self, rng='default'):
        r"""
        Set the gsl random number generator to be one of ``default``,
        ``luxury``, or ``taus``.

        EXAMPLES::

            sage: T = SphericalDistribution()
            sage: T.set_random_number_generator('default')
            sage: T.set_seed(0)
            sage: T.get_random_element()  # rel tol 4e-16
            (0.07961564104639995, -0.05237671627581255, 0.9954486572862178)
            sage: T.set_random_number_generator('luxury')
            sage: T.set_seed(0)
            sage: T.get_random_element()  # rel tol 4e-16
            (0.07961564104639995, -0.05237671627581255, 0.9954486572862178)
        """
        if rng == 'default':
            self.T = gsl_rng_default
        elif rng == 'luxury':
            self.T = gsl_rng_ranlxd2
        elif rng == 'taus':
            self.T = gsl_rng_taus2
        else:
            raise TypeError("Not a valid random number generator")

    def __dealloc__(self):
        if self.r != NULL:
            gsl_rng_free(self.r)
        sig_free(self.vec)

    def get_random_element(self):
        r"""
        Get a random sample from the probability distribution.

        EXAMPLES::

            sage: T = SphericalDistribution(seed=0)
            sage: T.get_random_element()  # rel tol 4e-16
            (0.07961564104639995, -0.05237671627581255, 0.9954486572862178)
        """
        cdef int i
        v = [0]*self.dimension
        gsl_ran_dir_nd(self.r, self.dimension, self.vec)
        for i in range(self.dimension):
            v[i] = self.vec[i]
        return vector(sage.rings.real_double.RDF, v)  # This could be made more efficient by directly constructing the vector, TODO.

    def reset_distribution(self):
        r"""
        This method resets the distribution.

        EXAMPLES::

            sage: T = SphericalDistribution(seed=0)
            sage: [T.get_random_element() for _ in range(4)]  # rel tol 4e-16
            [(0.07961564104639995, -0.05237671627581255, 0.9954486572862178),
             (0.4123599490593727, 0.5606817859360097, -0.7180495855658982),
             (-0.9619860891623148, -0.2726473494040498, -0.015690351211529927),
             (0.5674297579435619, -0.011206783800420301, -0.8233455397322326)]
            sage: T.reset_distribution()
            sage: [T.get_random_element() for _ in range(4)]  # rel tol 4e-16
            [(0.07961564104639995, -0.05237671627581255, 0.9954486572862178),
             (0.4123599490593727, 0.5606817859360097, -0.7180495855658982),
             (-0.9619860891623148, -0.2726473494040498, -0.015690351211529927),
             (0.5674297579435619, -0.011206783800420301, -0.8233455397322326)]
        """
        if self.r != NULL:
            gsl_rng_free(self.r)
        self.r = gsl_rng_alloc(self.T)
        self.set_seed(self.seed)
#        gsl_rng_env_setup()

cdef class RealDistribution(ProbabilityDistribution):
    r"""
    The :class:`RealDistribution` class provides a number of routines for sampling
    from and analyzing and visualizing probability distributions.
    For precise definitions of the distributions and their parameters
    see the gsl reference manuals chapter on random number generators
    and probability distributions.

    EXAMPLES:

    Uniform distribution on the interval ``[a, b]``::

        sage: a = 0
        sage: b = 2
        sage: T = RealDistribution('uniform', [a, b])
        sage: a <= T.get_random_element() <= b
        True
        sage: T.distribution_function(0)
        0.5
        sage: T.cum_distribution_function(1)
        0.5
        sage: T.cum_distribution_function_inv(.5)
        1.0

    The gaussian distribution takes 1 parameter ``sigma``. The standard
    gaussian distribution has ``sigma = 1``::

        sage: sigma = 1
        sage: T = RealDistribution('gaussian', sigma)
        sage: s = T.get_random_element()
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        0.3989422804014327
        sage: T.cum_distribution_function(1)
        0.8413447460685429
        sage: T.cum_distribution_function_inv(.5)
        0.0

    The rayleigh distribution has 1 parameter ``sigma``::

        sage: sigma = 3
        sage: T = RealDistribution('rayleigh', sigma)
        sage: s = T.get_random_element()
        sage: s >= 0
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        0.0
        sage: T.cum_distribution_function(1)
        0.054040531093234534
        sage: T.cum_distribution_function_inv(.5)
        3.532230067546424...

    The lognormal distribution has two parameters ``sigma``
    and ``zeta``::

        sage: zeta = 0
        sage: sigma = 1
        sage: T = RealDistribution('lognormal', [zeta, sigma])
        sage: s = T.get_random_element()
        sage: s >= 0
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        0.0
        sage: T.cum_distribution_function(1)
        0.5
        sage: T.cum_distribution_function_inv(.5)
        1.0

    The pareto distribution has two parameters ``a``, and ``b``::

        sage: a = 1
        sage: b = 1
        sage: T = RealDistribution('pareto', [a, b])
        sage: s = T.get_random_element()
        sage: s >= b
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        0.0
        sage: T.cum_distribution_function(1)
        0.0
        sage: T.cum_distribution_function_inv(.5)
        2.0

    The t-distribution has one parameter ``nu``::

        sage: nu = 1
        sage: T = RealDistribution('t', nu)
        sage: s = T.get_random_element()
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)      # rel tol 1e-15
        0.3183098861837906
        sage: T.cum_distribution_function(1)  # rel tol 1e-15
        0.75
        sage: T.cum_distribution_function_inv(.5)
        0.0

    The F-distribution has two parameters ``nu1`` and ``nu2``::

        sage: nu1 = 9; nu2 = 17
        sage: F = RealDistribution('F', [nu1,nu2])
        sage: s = F.get_random_element()
        sage: s >= 0
        True
        sage: s.parent()
        Real Double Field
        sage: F.distribution_function(1)  # rel tol 1e-14
        0.6695025505192798
        sage: F.cum_distribution_function(3.68)  # rel tol 1e-14
        0.9899717772300652
        sage: F.cum_distribution_function_inv(0.99)  # rel tol 1e-14
        3.682241524045864

    The chi-squared distribution has one parameter ``nu``::

        sage: nu = 1
        sage: T = RealDistribution('chisquared', nu)
        sage: s = T.get_random_element()
        sage: s >= 0
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        +infinity
        sage: T.cum_distribution_function(1)  # rel tol 1e-14
        0.6826894921370856
        sage: T.cum_distribution_function_inv(.5)  # rel tol 1e-14
        0.45493642311957305

    The exponential power distribution has two parameters ``a`` and
    ``b``::

        sage: a = 1
        sage: b = 2.5
        sage: T = RealDistribution('exppow', [a, b])
        sage: s = T.get_random_element()
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)  # rel tol 1e-14
        0.5635302489930136
        sage: T.cum_distribution_function(1)  # rel tol 1e-14
        0.940263052542855

    The beta distribution has two parameters ``a`` and ``b``::

        sage: a = 2
        sage: b = 2
        sage: T = RealDistribution('beta', [a, b])
        sage: s = T.get_random_element()
        sage: 0 <= s <= 1
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        0.0
        sage: T.cum_distribution_function(1)
        1.0

    The exponential distribution has one parameter ``mu``::

        sage: mu = 2
        sage: T = RealDistribution('exponential', mu)
        sage: s = T.get_random_element()
        sage: 0 <= s
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        0.5

    The gamma distribution has two parameters ``a`` and ``b``::

        sage: a = 2
        sage: b = 2
        sage: T = RealDistribution('gamma', [a, b])
        sage: s = T.get_random_element()
        sage: 0 <= s
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        0.0

    The weibull distribution has two parameters ``a`` and ``b``::

        sage: a = 1
        sage: b = 1
        sage: T = RealDistribution('weibull', [a, b])
        sage: s = T.get_random_element()
        sage: s >= 0
        True
        sage: s.parent()
        Real Double Field
        sage: T.distribution_function(0)
        1.0
        sage: T.cum_distribution_function(1)
        0.6321205588285577
        sage: T.cum_distribution_function_inv(.5)
        0.6931471805599453

    It is possible to select which random number generator drives the
    sampling as well as the seed.  The default is the Mersenne
    twister. Also available are the RANDLXS algorithm and the
    Tausworthe generator (see the gsl reference manual for more
    details). These are all supposed to be simulation quality
    generators. For RANDLXS use ``rng='luxury'`` and for
    tausworth use ``rng='taus'``::

         sage: T = RealDistribution('gaussian', 1, rng='luxury', seed=10)

    To change the seed at a later time use ``set_seed``::

         sage: T.set_seed(100)

    TESTS:

    Make sure that repeated initializations are randomly seeded
    (:issue:`9770`)::

        sage: Xs = [RealDistribution('gaussian', 1).get_random_element() for _ in range(1000)]
        sage: len(set(Xs)) > 2^^32
        True
    """
    cdef const gsl_rng_type *T
    cdef gsl_rng *r
    cdef int distribution_type
    cdef double* parameters
    cdef long int seed
    cdef object name
    # cdef double (*generator_1)(gsl_rng*)
    # cdef double (*generator_2)(gsl_rng*, double)
    # cdef _get_random_element_c(self)

    def __init__(self, type='uniform', parameters=None,
                 rng='default', seed=None):
        r"""
        EXAMPLES::

            sage: T = RealDistribution('gaussian', 1, seed=0)
            sage: T.get_random_element()  # rel tol 4e-16
            0.13391860811867587

        TESTS:

        Until :issue:`15089` a value of the ``seed`` keyword
        besides ``None`` was ignored. We check here that setting
        a seed is effective. ::

            sage: T = RealDistribution("beta",[1.6,4.3], seed=876)
            sage: one = [T.get_random_element() for _ in range(10)]
            sage: T = RealDistribution("beta",[1.6,4.3], seed=876)
            sage: two = [T.get_random_element() for _ in range(10)]
            sage: T = RealDistribution("beta",[1.6,4.3], seed=123)
            sage: three = [T.get_random_element() for _ in range(10)]
            sage: one == two
            True
            sage: one == three
            False
        """
        gsl_rng_env_setup()
        self.parameters = NULL
        self.set_random_number_generator(rng)
        self.r = gsl_rng_alloc(self.T)
        if seed is None:
            seed = random.randint(1, 2**31)
        self.set_seed(seed)
        self.name = " "
        self.set_distribution(type, parameters)

    def set_seed(self, seed):
        r"""
        Set the seed for the underlying random number generator.

        EXAMPLES::

            sage: T = RealDistribution('gaussian', 1, rng='luxury', seed=10)
            sage: T.set_seed(100)
        """

        gsl_rng_set(self.r, seed)
        self.seed = seed

    def set_random_number_generator(self, rng='default'):
        r"""
        Set the gsl random number generator to be one of ``'default'``,
        ``'luxury'``, or ``'taus'``.

        EXAMPLES::

            sage: T = SphericalDistribution()
            sage: T.set_random_number_generator('default')
            sage: T.set_seed(0)
            sage: T.get_random_element()  # rel tol 4e-16
            (0.07961564104639995, -0.05237671627581255, 0.9954486572862178)
            sage: T.set_random_number_generator('luxury')
            sage: T.set_seed(0)
            sage: T.get_random_element()  # rel tol 4e-16
            (0.07961564104639995, -0.05237671627581255, 0.9954486572862178)
        """
        if rng == 'default':
            self.T = gsl_rng_default
        elif rng == 'luxury':
            self.T = gsl_rng_ranlxd2
        elif rng == 'taus':
            self.T = gsl_rng_taus2
        else:
            raise TypeError("Not a valid random number generator")

    def __dealloc__(self):
        if self.r != NULL:
            gsl_rng_free(self.r)
        sig_free(self.parameters)

    def __str__(self):
        r"""
        Return the name of the current distribution.

        EXAMPLES::

            sage: T = RealDistribution('gaussian', 1)
            sage: str(T)
            'gaussian'
            sage: T = RealDistribution('beta', [2, 2])
            sage: str(T)
            'beta'
        """
        return self.name

    def get_random_element(self):
        r"""
        Get a random sample from the probability distribution.

        EXAMPLES::

            sage: T = RealDistribution('gaussian', 1, seed=0)
            sage: T.get_random_element()  # rel tol 4e-16
            0.13391860811867587
        """
        cdef double result
        if self.distribution_type == uniform:
            result = gsl_ran_flat(self.r, self.parameters[0], self.parameters[1])
#            result = gsl_rng_uniform(self.r)
        elif self.distribution_type == gaussian:
            result = gsl_ran_gaussian(self.r, self.parameters[0])
        elif self.distribution_type == rayleigh:
            result = gsl_ran_rayleigh(self.r, self.parameters[0])
        elif self.distribution_type == lognormal:
            result = gsl_ran_lognormal(self.r, self.parameters[0], self.parameters[1])
        elif self.distribution_type == pareto:
            result = gsl_ran_pareto(self.r, self.parameters[0], self.parameters[1])
        elif self.distribution_type == t:
            result = gsl_ran_tdist(self.r, self.parameters[0])
        elif self.distribution_type == F:
            result = gsl_ran_fdist(self.r, self.parameters[0], self.parameters[1])
        elif self.distribution_type == chisquared:
            result = gsl_ran_chisq(self.r, self.parameters[0])
        elif self.distribution_type == exppow:
            result = gsl_ran_exppow(self.r, self.parameters[0], self.parameters[1])
        elif self.distribution_type == weibull:
            result = gsl_ran_weibull(self.r, self.parameters[0], self.parameters[1])
        elif self.distribution_type == beta:
            result = gsl_ran_beta(self.r, self.parameters[0], self.parameters[1])
        elif self.distribution_type == exponential:
            result = gsl_ran_exponential(self.r, self.parameters[0])
        elif self.distribution_type == gamma:
            result = gsl_ran_gamma(self.r, self.parameters[0], self.parameters[1])
        else:
            raise TypeError("Not a supported probability distribution")

        return sage.rings.real_double.RDF(result)

    def set_distribution(self, name='uniform', parameters=None):
        r"""
        This method can be called to change the current probability distribution.

        EXAMPLES::

            sage: T = RealDistribution('gaussian', 1)
            sage: T.set_distribution('gaussian', 1)
            sage: T.set_distribution('pareto', [0, 1])
        """
        sig_free(self.parameters)

        if parameters is None:
            parameters = []

        if name == 'uniform':
            self.distribution_type = uniform
            for x in parameters:
                try:
                    float(x)
                except Exception:
                    raise TypeError("Uniform distribution requires parameters coercible to float")
            self.parameters = <double*>sig_malloc(sizeof(double)*2)
            self.parameters[0] = parameters[0]
            self.parameters[1] = parameters[1]
        elif name == 'gaussian':
            try:
                float(parameters)
            except Exception:
                raise TypeError("gaussian distribution requires parameter sigma coercible to float")
            self.parameters = <double*>sig_malloc(sizeof(double))
            self.parameters[0] = float(parameters)
            self.distribution_type = gaussian
        elif name == 'pareto':
            if len(parameters) != 2:
                raise TypeError("pareto distribution has two parameters")
            try:
                map(float, parameters)
            except Exception:
                raise TypeError("parameters must be coercible to float")
            self.parameters = <double*>sig_malloc(sizeof(double)*2)
            self.parameters[0] = float(parameters[0])
            self.parameters[1] = float(parameters[1])
            self.distribution_type = pareto
        elif name == 'rayleigh':
            try:
                float(parameters)
            except Exception:
                raise TypeError("rayleigh distribution requires parameter sigma coercible to float")
            self.parameters = <double*>sig_malloc(sizeof(double))
            self.parameters[0] = float(parameters)
            self.distribution_type = rayleigh
        elif name == 'lognormal':
            if len(parameters) != 2:
                raise TypeError("Lognormal distribution requires two parameters")
            for x in parameters:
                try:
                    float(x)
                except Exception:
                    raise TypeError("Lognormal distribution requires real parameters")
            self.parameters = <double*>sig_malloc(sizeof(double)*2)
            self.parameters[0] = float(parameters[0])
            self.parameters[1] = float(parameters[1])
            self.distribution_type = lognormal
        elif name == 't':
            try:
                float(parameters)
            except Exception:
                raise TypeError("parameter to t distribution must be coercible to float")
            self.parameters = <double*>sig_malloc(sizeof(double))
            self.parameters[0] = float(parameters)
            self.distribution_type = t
        elif name == 'F':
            if len(parameters) != 2:
                raise TypeError("F-distribution requires two real parameters")
            try:
                map(float, parameters)
            except Exception:
                raise TypeError("F-distribution requires real parameters")
            self.parameters = <double *>sig_malloc(sizeof(double)*2)
            self.parameters[0] = float(parameters[0])
            self.parameters[1] = float(parameters[1])
            self.distribution_type = F
        elif name == 'chisquared':
            try:
                float(parameters)
            except Exception:
                raise TypeError("parameters to t distribution must be coercible to float")
            self.parameters = <double *>sig_malloc(sizeof(double))
            self.parameters[0] = float(parameters)
            self.distribution_type = chisquared
        elif name == 'exppow':
            if len(parameters) != 2:
                raise TypeError("exponential power distribution requires two parameters")
            for x in parameters:
                try:
                    float(x)
                except Exception:
                    raise TypeError("exponential power distribution requires real parameters")
            self.parameters = <double*>sig_malloc(sizeof(double)*2)
            self.parameters[0] = float(parameters[0])
            self.parameters[1] = float(parameters[1])
            self.distribution_type = exppow
        elif name == 'weibull':
            if len(parameters) != 2:
                raise TypeError("weibull distribution requires two real parameters")
            try:
                map(float, parameters)
            except Exception:
                raise TypeError("weibull distribution requires real parameters")
            self.parameters = <double *>sig_malloc(sizeof(double)*2)
            self.parameters[0] = float(parameters[0])
            self.parameters[1] = float(parameters[1])
            self.distribution_type = weibull
        elif name == 'beta':
            if len(parameters) != 2:
                raise TypeError("beta distribution requires two real parameters")
            try:
                map(float, parameters)
            except Exception:
                raise TypeError("beta distribution requires real parameters")
            self.parameters = <double *>sig_malloc(sizeof(double)*2)
            self.parameters[0] = float(parameters[0])
            self.parameters[1] = float(parameters[1])
            self.distribution_type = beta
        elif name == 'exponential':
            try:
                float(parameters)
            except Exception:
                raise TypeError("exponential distribution requires parameter mu coercible to float")
            self.parameters = <double*>sig_malloc(sizeof(double))
            self.parameters[0] = float(parameters)
            self.distribution_type = exponential
        elif name == 'gamma':
            if len(parameters) != 2:
                raise TypeError("gamma distribution requires two real parameters")
            try:
                map(float, parameters)
            except Exception:
                raise TypeError("gamma distribution requires real parameters")
            self.parameters = <double *>sig_malloc(sizeof(double)*2)
            self.parameters[0] = float(parameters[0])
            self.parameters[1] = float(parameters[1])
            self.distribution_type = gamma
        else:
            raise TypeError("Not a supported probability distribution")

        self.name = name

    # def _get_random_element_c():

    def reset_distribution(self):
        r"""
        Reset the distribution.

        EXAMPLES::

            sage: T = RealDistribution('gaussian', 1, seed=10)
            sage: [T.get_random_element() for _ in range(10)]  # rel tol 4e-16
            [-0.7460999595745819, -0.004644606626413462, -0.8720538317207641, 0.6916259921666037, 2.67668674666043, 0.6325002813661014, -0.7974263521959355, -0.5284976893366636, 1.1353119849528792, 0.9912505673230749]
            sage: T.reset_distribution()
            sage: [T.get_random_element() for _ in range(10)]  # rel tol 4e-16
            [-0.7460999595745819, -0.004644606626413462, -0.8720538317207641, 0.6916259921666037, 2.67668674666043, 0.6325002813661014, -0.7974263521959355, -0.5284976893366636, 1.1353119849528792, 0.9912505673230749]
        """
        if self.r != NULL:
            gsl_rng_free(self.r)
        self.r = gsl_rng_alloc(self.T)
        self.set_seed(self.seed)
#        gsl_rng_env_setup()

    def distribution_function(self, x):
        r"""
        Evaluate the distribution function of the
        probability distribution at ``x``.

        EXAMPLES::

            sage: T = RealDistribution('uniform', [0, 2])
            sage: T.distribution_function(0)
            0.5
            sage: T.distribution_function(1)
            0.5
            sage: T.distribution_function(1.5)
            0.5
            sage: T.distribution_function(2)
            0.0
        """
        if self.distribution_type == uniform:
            return sage.rings.real_double.RDF(gsl_ran_flat_pdf(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == gaussian:
            return sage.rings.real_double.RDF(gsl_ran_gaussian_pdf(x, self.parameters[0]))
        elif self.distribution_type == rayleigh:
            return sage.rings.real_double.RDF(gsl_ran_rayleigh_pdf(x, self.parameters[0]))
        elif self.distribution_type == lognormal:
            return sage.rings.real_double.RDF(gsl_ran_lognormal_pdf(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == pareto:
            return sage.rings.real_double.RDF(gsl_ran_pareto_pdf(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == t:
            return sage.rings.real_double.RDF(gsl_ran_tdist_pdf(x, self.parameters[0]))
        elif self.distribution_type == F:
            return sage.rings.real_double.RDF(gsl_ran_fdist_pdf(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == chisquared:
            return sage.rings.real_double.RDF(gsl_ran_chisq_pdf(x, self.parameters[0]))
        elif self.distribution_type == exppow:
            return sage.rings.real_double.RDF(gsl_ran_exppow_pdf(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == weibull:
            return sage.rings.real_double.RDF(gsl_ran_weibull_pdf(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == beta:
            return sage.rings.real_double.RDF(gsl_ran_beta_pdf(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == exponential:
            return sage.rings.real_double.RDF(gsl_ran_exponential_pdf(x, self.parameters[0]))
        elif self.distribution_type == gamma:
            return sage.rings.real_double.RDF(gsl_ran_gamma_pdf(x, self.parameters[0], self.parameters[1]))
        else:
            raise TypeError("Not a supported probability distribution")

    def cum_distribution_function(self, x):
        r"""
        Evaluate the cumulative distribution function of
        the probability distribution at ``x``.

        EXAMPLES::

            sage: T = RealDistribution('uniform', [0, 2])
            sage: T.cum_distribution_function(1)
            0.5
        """
        if self.distribution_type == uniform:
            return sage.rings.real_double.RDF(gsl_cdf_flat_P(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == gaussian:
            return sage.rings.real_double.RDF(gsl_cdf_gaussian_P(x, self.parameters[0]))
        elif self.distribution_type == rayleigh:
            return sage.rings.real_double.RDF(gsl_cdf_rayleigh_P(x, self.parameters[0]))
        elif self.distribution_type == lognormal:
            return sage.rings.real_double.RDF(gsl_cdf_lognormal_P(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == pareto:
            return sage.rings.real_double.RDF(gsl_cdf_pareto_P(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == t:
            return sage.rings.real_double.RDF(gsl_cdf_tdist_P(x, self.parameters[0]))
        elif self.distribution_type == F:
            return sage.rings.real_double.RDF(gsl_cdf_fdist_P(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == chisquared:
            return sage.rings.real_double.RDF(gsl_cdf_chisq_P(x, self.parameters[0]))
        elif self.distribution_type == exppow:
            return sage.rings.real_double.RDF(gsl_cdf_exppow_P(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == weibull:
            return sage.rings.real_double.RDF(gsl_cdf_weibull_P(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == beta:
            return sage.rings.real_double.RDF(gsl_cdf_beta_P(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == exponential:
            return sage.rings.real_double.RDF(gsl_cdf_exponential_P(x, self.parameters[0]))
        elif self.distribution_type == gamma:
            return sage.rings.real_double.RDF(gsl_cdf_gamma_P(x, self.parameters[0], self.parameters[1]))
        else:
            raise TypeError("Not a supported probability distribution")

    def cum_distribution_function_inv(self, x):
        r"""
        Evaluate the inverse of the cumulative distribution
        distribution function of the probability distribution at ``x``.

        EXAMPLES::

            sage: T = RealDistribution('uniform', [0, 2])
            sage: T.cum_distribution_function_inv(.5)
            1.0
        """
        if self.distribution_type == uniform:
            return sage.rings.real_double.RDF(gsl_cdf_flat_Pinv(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == gaussian:
            return sage.rings.real_double.RDF(gsl_cdf_gaussian_Pinv(x, self.parameters[0]))
        elif self.distribution_type == rayleigh:
            return sage.rings.real_double.RDF(gsl_cdf_rayleigh_Pinv(x, self.parameters[0]))
        elif self.distribution_type == lognormal:
            return sage.rings.real_double.RDF(gsl_cdf_lognormal_Pinv(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == pareto:
            return sage.rings.real_double.RDF(gsl_cdf_pareto_Pinv(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == t:
            return sage.rings.real_double.RDF(gsl_cdf_tdist_Pinv(x, self.parameters[0]))
        elif self.distribution_type == F:
            return sage.rings.real_double.RDF(gsl_cdf_fdist_Pinv(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == chisquared:
            return sage.rings.real_double.RDF(gsl_cdf_chisq_Pinv(x, self.parameters[0]))
        elif self.distribution_type == exppow:
            raise NotImplementedError("gsl does not provide inverse for exponential power")
#            return sage.rings.real_double.RDF(gsl_cdf_exppow_Pinv(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == weibull:
            return sage.rings.real_double.RDF(gsl_cdf_weibull_Pinv(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == beta:
            return sage.rings.real_double.RDF(gsl_cdf_beta_Pinv(x, self.parameters[0], self.parameters[1]))
        elif self.distribution_type == exponential:
            return sage.rings.real_double.RDF(gsl_cdf_exponential_Pinv(x, self.parameters[0]))
        elif self.distribution_type == gamma:
            return sage.rings.real_double.RDF(gsl_cdf_gamma_Pinv(x, self.parameters[0], self.parameters[1]))
        else:
            raise TypeError("Not a supported probability distribution")

    def plot(self, *args, **kwds):
        r"""
        Plot the distribution function for the probability
        distribution. Parameters to :func:`sage.plot.plot.plot` can be
        passed through ``*args`` and ``**kwds``.

        EXAMPLES::

            sage: T = RealDistribution('uniform', [0, 2])
            sage: P = T.plot()
        """
        from sage.plot.plot import plot
        return plot(self.distribution_function, *args, **kwds)


cdef class GeneralDiscreteDistribution(ProbabilityDistribution):
    r"""
    Create a discrete probability distribution.

    INPUT:

    - ``P`` -- list of probabilities; the list will automatically be
      normalised if ``sum(P)`` is not equal to 1

    - ``rng`` -- (optional) random number generator to use; may be
      one of ``'default'``, ``'luxury'``, or ``'taus'``

    - ``seed`` -- (optional) seed to use with the random number
      generator

    OUTPUT: a probability distribution where the probability of selecting
    ``x`` is ``P[x]``.

    EXAMPLES:

    Construct a ``GeneralDiscreteDistribution`` with the probability
    distribution `P` where `P(0) = 0.3`, `P(1) = 0.4`, `P(2) = 0.3`::

        sage: P = [0.3, 0.4, 0.3]
        sage: X = GeneralDiscreteDistribution(P)
        sage: X.get_random_element() in (0, 1, 2)
        True

    Checking the distribution of samples::

        sage: P = [0.3, 0.4, 0.3]
        sage: counts = [0] * len(P)
        sage: X = GeneralDiscreteDistribution(P)
        sage: nr_samples = 10000
        sage: for _ in range(nr_samples):
        ....:     counts[X.get_random_element()] += 1
        sage: [1.0*x/nr_samples for x in counts]  # abs tol 3e-2
        [0.3, 0.4, 0.3]

    The distribution probabilities will automatically be normalised::

        sage: P = [0.1, 0.3]
        sage: X = GeneralDiscreteDistribution(P, seed=0)
        sage: counts = [0, 0]
        sage: for _ in range(10000):
        ....:     counts[X.get_random_element()] += 1
        sage: float(counts[1]/counts[0])
        3.042037186742118

    TESTS:

    Make sure that repeated initializations are randomly seeded
    (:issue:`9770`)::

        sage: P = [0.001] * 1000
        sage: Xs = [GeneralDiscreteDistribution(P).get_random_element() for _ in range(1000)]
        sage: len(set(Xs)) > 2^^32
        True

    The distribution probabilities must be nonnegative::

        sage: GeneralDiscreteDistribution([0.1, -0.1])
        Traceback (most recent call last):
        ...
        ValueError: The distribution probabilities must be nonnegative
    """
    cdef const gsl_rng_type * T
    cdef gsl_rng * r
    cdef gsl_ran_discrete_t *dist
    cdef long seed

    def __init__(self, P, rng='default', seed=None):
        r"""
        Given a list of probabilities P construct an instance of a gsl
        discrete random variable generator.

        EXAMPLES::

            sage: P = [0.3, 0.4, 0.3]
            sage: X = GeneralDiscreteDistribution(P)
            sage: assert X.get_random_element() in range(len(P))

        TESTS:

        Until :issue:`15089` a value of the ``seed`` keyword
        besides ``None`` was ignored. We check here that setting
        a seed is effective. ::

            sage: P = [0.2, 0.3, 0.1, 0.4]
            sage: T = GeneralDiscreteDistribution(P, seed=876)
            sage: one = [T.get_random_element() for _ in range(50)]
            sage: T = GeneralDiscreteDistribution(P, seed=876)
            sage: two = [T.get_random_element() for _ in range(50)]
            sage: T = GeneralDiscreteDistribution(P, seed=123)
            sage: three = [T.get_random_element() for _ in range(50)]
            sage: one == two
            True
            sage: one == three
            False

        Testing that :issue:`24416` is fixed for when entries are larger
        than `2^{1024}`::

            sage: from collections import Counter
            sage: X = GeneralDiscreteDistribution([1,2,2^1024])
            sage: Counter(X.get_random_element() for _ in range(100))
            Counter({2: 100})
        """
        gsl_rng_env_setup()
        self.set_random_number_generator(rng)
        self.r = gsl_rng_alloc(self.T)
        if seed is None:
            seed = random.randint(1, 2**31)
        self.set_seed(seed)

        cdef int n
        n = len(P)

        s = sum(P)
        if s != 1:
            P = [p/s for p in P]

        cdef double *P_vec
        P_vec = <double *> sig_malloc(n*(sizeof(double)))

        cdef int i
        for i in range(n):
            if P[i] < 0:
                raise ValueError("The distribution probabilities must "
                                 "be nonnegative")
            P_vec[i] = P[i]

        self.dist = gsl_ran_discrete_preproc(n, P_vec)

        sig_free(P_vec)

    def set_seed(self, seed):
        r"""
        Set the seed to be used by the random number generator.

        EXAMPLES::

            sage: X = GeneralDiscreteDistribution([0.3, 0.4, 0.3])
            sage: X.set_seed(1)
            sage: X.get_random_element()
            1
        """
        gsl_rng_set(self.r, seed)
        self.seed = seed

    def set_random_number_generator(self, rng='default'):
        r"""
        Set the random number generator to be used by gsl.

        EXAMPLES::

            sage: X = GeneralDiscreteDistribution([0.3, 0.4, 0.3])
            sage: X.set_random_number_generator('taus')
        """
        if rng == 'default':
            self.T = gsl_rng_default
        elif rng == 'luxury':
            self.T = gsl_rng_ranlxd2
        elif rng == 'taus':
            self.T = gsl_rng_taus2
        else:
            raise TypeError("Not a valid random number generator")

    def __dealloc__(self):
        if self.r != NULL:
            gsl_rng_free(self.r)

        if self.dist != NULL:
            gsl_ran_discrete_free(self.dist)

    def get_random_element(self):
        r"""
        Get a random sample from the probability distribution.

        EXAMPLES::

            sage: P = [0.3, 0.4, 0.3]
            sage: X = GeneralDiscreteDistribution(P)
            sage: all(X.get_random_element() in (0,1,2) for _ in range(10))
            True
            sage: isinstance(X.get_random_element(), sage.rings.integer.Integer)
            True
        """
        return sage.rings.integer.Integer(gsl_ran_discrete(self.r, self.dist))

    def reset_distribution(self):
        r"""
        This method resets the distribution.

        EXAMPLES::

            sage: T = GeneralDiscreteDistribution([0.1, 0.3, 0.6])
            sage: T.set_seed(0)
            sage: [T.get_random_element() for _ in range(10)]
            [2, 2, 2, 2, 2, 1, 2, 2, 1, 2]
            sage: T.reset_distribution()
            sage: [T.get_random_element() for _ in range(10)]
            [2, 2, 2, 2, 2, 1, 2, 2, 1, 2]
        """
        if self.r != NULL:
            gsl_rng_free(self.r)
        self.r = gsl_rng_alloc(self.T)
        self.set_seed(self.seed)



r"""
Sage Interacts

Sage interacts are applications of the `@interact decorator <../../sagenb/notebook/interact.html>`_.
They are conveniently accessible in the Sage notebook via ``interacts.[TAB].[TAB]()``.
The first ``[TAB]`` lists categories and the second ``[TAB]`` reveals the interact examples.

EXAMPLES:

Invoked in the notebook, the following command will produce the fully formatted
interactive mathlet.  In the command line, it will simply return the underlying
HTML and Sage code which creates the mathlet::

    sage: interacts.calculus.taylor_polynomial()
    ...Interactive function <function taylor_polynomial at ...> with 3 widgets
      title: HTMLText(value='<h2>Taylor polynomial</h2>')
      f: EvalText(value='e^(-x)*sin(x)', description='$f(x)=$', layout=Layout(max_width='81em'))
      order: SelectionSlider(description='order', options=(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12), value=1)

AUTHORS:

- William Stein
- Harald Schilly, Robert Marik (2011-01-16): added many examples (#9623)
  partially based on work by Lauri Ruotsalainen
"""

# *****************************************************************************
#       Copyright (C) 2009 William Stein <wstein@gmail.com>
#       Copyright (C) 2011 Harald Schilly <harald.schilly@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#                  https://www.gnu.org/licenses/
# *****************************************************************************

from collections.abc import Callable
from typing import Any

from sage.arith.misc import factor
from sage.arith.srange import srange
from sage.calculus.expr import symbolic_expression
from sage.calculus.functional import derivative
from sage.calculus.integration import numerical_integral as integral_numerical
from sage.ext.fast_callable import fast_callable
from sage.functions.log import exp
from sage.functions.trig import acos, cos, sin, tan
from sage.misc.decorators import sage_wraps
from sage.misc.functional import N, sqrt
from sage.misc.latex import latex
from sage.misc.lazy_import import lazy_import
from sage.misc.sage_eval import sage_eval
from sage.misc.table import table

lazy_import("sage.plot.circle", "circle")
lazy_import("sage.plot.complex_plot", "complex_plot")
lazy_import("sage.plot.disk", "disk")
lazy_import("sage.plot.graphics", "Graphics")
lazy_import("sage.plot.line", ["line", "line2d"])
lazy_import("sage.plot.matrix_plot", "matrix_plot")
lazy_import("sage.plot.plot", ["graphics_array", "parametric_plot", "plot"])
lazy_import("sage.plot.point", ["point", "points"])
lazy_import("sage.plot.polygon", "polygon2d")
lazy_import("sage.plot.text", "text")
from sage.repl.rich_output.pretty_print import pretty_print, show
from sage.rings.complex_double import CDF
from sage.rings.integer import Integer
from sage.symbolic.constants import pi
from sage.symbolic.relation import solve
from sage.symbolic.ring import SR

x = SR.var('x')

# It is important that this file is lazily imported for this to work
from sage.repl.user_globals import get_global

assert get_global  # to suppress pyflakes warning

from sage.repl.ipython_kernel.all_jupyter import (
    checkbox,
    input_box,
    input_grid,
    interact,
    range_slider,
    selector,
    slider,
    text_control,
)


def library_interact(
    decorator_target: Callable[..., Any] | None = None, **widgets: Callable[..., Any]
):
    r"""
    This is a decorator for using interacts in the Sage library.

    This is essentially a wrapper around the ``interact`` function.

    INPUT:

    - ``**widgets`` -- keyword arguments that are passed to the
      ``interact`` function to create the widgets. Each value must
      be a callable that returns a widget.

    EXAMPLES::

        sage: from sage.interacts.library import library_interact
        sage: @library_interact(n=lambda: slider(-5, 15, None, 5))
        ....: def f(n):
        ....:     print(n)
        sage: f()  # an interact appears if using the notebook, else code
        ...Interactive function <function f at ...> with 1 widget
        n: TransformIntSlider(value=5, description='n', max=15, min=-5)

    TESTS:

    Backwards compatibility::

        sage: from sage.interacts.library import library_interact
        sage: @library_interact
        ....: def f(n=slider(-5, 15, None, 5)):
        ....:     print(n)
        doctest:warning
        ...
        DeprecationWarning: Use decorator factory @library_interact(widgets) instead of @library_interact without any arguments.
        See https://github.com/sagemath/sage/issues/33382 for details.
        sage: f()  # an interact appears if using the notebook, else code
        ...Interactive function <function f at ...> with 1 widget
        n: TransformIntSlider(value=5, description='n', max=15, min=-5)

    .. NOTE::

        We use a callable to construct the widget so that the widget is only
        initialized when the function is called and not upon loading the module.
    """

    def decorator(f: Callable[..., Any]):
        @sage_wraps(f)
        def library_wrapper():
            widgets_constructed = {
                key: constructor() for key, constructor in widgets.items()
            }
            # This will display the interact, no need to return anything
            interact(f, **widgets_constructed)

        # Store the widget constructors for later use in tests
        library_wrapper._widgets = widgets
        return library_wrapper

    if decorator_target is None:
        return decorator
    from sage.misc.superseded import deprecation

    deprecation(
        33382,
        "Use decorator factory @library_interact(widgets) instead of @library_interact without any arguments.",
    )
    return decorator(decorator_target)


def html(obj):
    r"""
    Shorthand to pretty print HTML.

    EXAMPLES::

        sage: from sage.interacts.library import html
        sage: html("<h1>Hello world</h1>")
        <h1>Hello world</h1>
    """
    from sage.misc.html import html
    pretty_print(html(obj))


@library_interact(n=lambda: slider(range(10)), m=lambda: slider(range(10)))
def demo(n: int, m: int):
    r"""
    This is a demo interact that sums two numbers.

    INPUT:

    - ``n`` -- integer
    - ``m`` -- integer

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.demo()
        ...Interactive function <function demo at ...> with 2 widgets
          n: SelectionSlider(description='n', options=(0, 1, 2, 3, 4, 5, 6, 7, 8, 9), value=0)
          m: SelectionSlider(description='m', options=(0, 1, 2, 3, 4, 5, 6, 7, 8, 9), value=0)
    """
    print(n + m)


@library_interact(
    title=lambda: text_control("<h2>Taylor polynomial</h2>"),
    f=lambda: input_box(sin(x) * exp(-x), label="$f(x)=$"),
    order=lambda: slider(range(1, 13)),
)
def taylor_polynomial(title, f, order: int):
    r"""
    Illustrate the Taylor polynomial approximation
    of various orders around `x=0`.

    INPUT:

    - ``f`` -- function expression
    - ``order`` -- integer

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.taylor_polynomial()
        ...Interactive function <function taylor_polynomial at ...> with 3 widgets
          title: HTMLText(value='<h2>Taylor polynomial</h2>')
          f: EvalText(value='e^(-x)*sin(x)', description='$f(x)=$', layout=Layout(max_width='81em'))
          order: SelectionSlider(description='order', options=(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12), value=1)
    """
    x0 = 0
    p = plot(f, (x, -1, 5), thickness=2)
    dot = point((x0, f(x=x0)), pointsize=80, rgbcolor=(1, 0, 0))
    ft = f.taylor(x, x0, order)
    pt = plot(ft, (-1, 5), color='green', thickness=2)
    html(r'$f(x)\;=\;%s$' % latex(f))
    html(r'$\hat{f}(x;%s)\;=\;%s+\mathcal{O}(x^{%s})$' % (x0, latex(ft),
                                                          order + 1))
    show(dot + p + pt, ymin=-.5, ymax=1)


@library_interact(
    title=lambda: text_control("<h2>Definite integral</h2>"),
    f=lambda: input_box(default='3*x', label="$f(x)=$"),
    g=lambda: input_box(default='x^2', label="$g(x)=$"),
    interval=lambda: range_slider(-10, 10, default=(0, 3), label='Interval'),
    x_range=lambda: range_slider(-10, 10, default=(0, 3), label="plot range (x)"),
    selection=lambda: selector(
        ["f", "g", "f and g", "f - g"], default="f and g", label="Select"
    ),
)
def definite_integral(title, f, g, interval, x_range, selection):
    r"""
    This is a demo interact for plotting the definite integral of a function
    based on work by Lauri Ruotsalainen, 2010.

    INPUT:

    - ``function`` -- input box, function in x
    - ``interval`` -- interval for the definite integral
    - ``x_range`` -- range slider for plotting range
    - ``selection`` -- selector on how to visualize the integrals

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.definite_integral()
        ...Interactive function <function definite_integral at ...> with 6 widgets
          title: HTMLText(value='<h2>Definite integral</h2>')
          f: EvalText(value='3*x', description='$f(x)=$', layout=Layout(max_width='81em'))
          g: EvalText(value='x^2', description='$g(x)=$', layout=Layout(max_width='81em'))
          interval: IntRangeSlider(value=(0, 3), description='Interval', max=10, min=-10)
          x_range: IntRangeSlider(value=(0, 3), description='plot range (x)', max=10, min=-10)
          selection: Dropdown(description='Select', index=2,
                              options=('f', 'g', 'f and g', 'f - g'), value='f and g')
    """
    x = SR.var('x')
    f = symbolic_expression(f).function(x)
    g = symbolic_expression(g).function(x)
    f_plot = Graphics()
    g_plot = Graphics()
    h_plot = Graphics()
    text = ""

    # Plot function f.
    if selection != "g":
        f_plot = plot(f(x), x, x_range, color='blue', thickness=1.5)

    # Color and calculate the area between f and the horizontal axis.
    if selection == "f" or selection == "f and g":
        f_plot += plot(f(x), x, interval, color='blue', fill=True, fillcolor='blue', fillalpha=0.15)
        text += r"$\int_{%.2f}^{%.2f}(\color{Blue}{f(x)})\,\mathrm{d}x=\int_{%.2f}^{%.2f}(%s)\,\mathrm{d}x=%.2f$" % (
            interval[0], interval[1],
            interval[0], interval[1],
            latex(f(x)),
            f(x).nintegrate(x, interval[0], interval[1])[0]
        )

    if selection == "f and g":
        text += r"<br/>"

    # Plot function g. Also color and calculate the area between g and the horizontal axis.
    if selection == "g" or selection == "f and g":
        g_plot = plot(g(x), x, x_range, color='green', thickness=1.5)
        g_plot += plot(g(x), x, interval, color='green', fill=True, fillcolor='yellow', fillalpha=0.5)
        text += r"$\int_{%.2f}^{%.2f}(\color{Green}{g(x)})\,\mathrm{d}x=\int_{%.2f}^{%.2f}(%s)\,\mathrm{d}x=%.2f$" % (
            interval[0], interval[1],
            interval[0], interval[1],
            latex(g(x)),
            g(x).nintegrate(x, interval[0], interval[1])[0]
        )

    # Plot function f-g. Also color and calculate the area between f-g and the horizontal axis.
    if selection == "f - g":
        g_plot = plot(g(x), x, x_range, color='green', thickness=1.5)
        g_plot += plot(g(x), x, interval, color='green', fill=f(x), fillcolor='red', fillalpha=0.15)
        h_plot = plot(f(x)-g(x), x, interval, color='red', thickness=1.5, fill=True, fillcolor='red', fillalpha=0.15)
        text = r"$\int_{%.2f}^{%.2f}(\color{Red}{f(x)-g(x)})\,\mathrm{d}x=\int_{%.2f}^{%.2f}(%s)\,\mathrm{d}x=%.2f$" % (
            interval[0], interval[1],
            interval[0], interval[1],
            latex(f(x)-g(x)),
            (f(x)-g(x)).nintegrate(x, interval[0], interval[1])[0]
        )

    show(f_plot + g_plot + h_plot, gridlines=True)
    html(text)


@library_interact(
    title=lambda: text_control("<h2>Derivative grapher</h2>"),
    function=lambda: input_box(default='x^5-3*x^3+1', label='Function:'),
    x_range=lambda: range_slider(-15, 15, 0.1, default=(-2, 2), label="Range (x)"),
    y_range=lambda: range_slider(-15, 15, 0.1, default=(-8, 6), label="Range (y)"),
)
def function_derivative(title, function, x_range, y_range):
    r"""
    This is a demo interact for plotting derivatives of a function based on work by
    Lauri Ruotsalainen, 2010.

    INPUT:

        - ``function`` -- input box, function in x
        - ``x_range`` -- range slider for plotting range
        - ``y_range`` -- range slider for plotting range

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.function_derivative()
        ...Interactive function <function function_derivative at ...> with 4 widgets
          title: HTMLText(value='<h2>Derivative grapher</h2>')
          function: EvalText(value='x^5-3*x^3+1', description='Function:', layout=Layout(max_width='81em'))
          x_range: FloatRangeSlider(value=(-2.0, 2.0), description='Range (x)', max=15.0, min=-15.0)
          y_range: FloatRangeSlider(value=(-8.0, 6.0), description='Range (y)', max=15.0, min=-15.0)
    """
    x = SR.var('x')
    f = symbolic_expression(function).function(x)
    df = derivative(f, x)
    ddf = derivative(df, x)
    plots = plot(f(x), x_range, thickness=1.5) + plot(df(x), x_range, color='green') + plot(ddf(x), x_range, color='red')
    if y_range == (0,0):
        show(plots, xmin=x_range[0], xmax=x_range[1])
    else:
        show(plots, xmin=x_range[0], xmax=x_range[1], ymin=y_range[0], ymax=y_range[1])

    html(r"<center>$\color{Blue}{f(x) = %s}$</center>" % latex(f(x)))
    html(r"<center>$\color{Green}{f'(x) = %s}$</center>" % latex(df(x)))
    html(r"<center>$\color{Red}{f''(x) = %s}$</center>" % latex(ddf(x)))


@library_interact(
    title=lambda: text_control("<h2>Difference quotient</h2>"),
    f=lambda: input_box(default='sin(x)', label="f(x)"),
    interval=lambda: range_slider(0, 10, 0.1, default=(0.0, 10.0), label='Range'),
    a=lambda: slider(0, 10, None, 5.5, label='$a$'),
    x0=lambda: slider(0, 10, None, 2.5, label="$x_0$ (start point)"),
)
def difference_quotient(title, f, interval, a, x0):
    r"""
    This is a demo interact for difference quotient based on work by
    Lauri Ruotsalainen, 2010.

    INPUT:

    - ``f`` -- input box, function in `x`
    - ``interval`` -- range slider for plotting
    - ``a`` -- slider for `a`
    - ``x0`` -- slider for starting point `x_0`

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.difference_quotient()
        ...Interactive function <function difference_quotient at ...> with 5 widgets
          title: HTMLText(value='<h2>Difference quotient</h2>')
          f: EvalText(value='sin(x)', description='f(x)', layout=Layout(max_width='81em'))
          interval: FloatRangeSlider(value=(0.0, 10.0), description='Range', max=10.0)
          a: IntSlider(value=5, description='$a$', max=10)
          x0: IntSlider(value=2, description='$x_0$ (start point)', max=10)
    """
    html('<h2>Difference Quotient</h2>')
    html('<div style="white-space: normal;">\
         <a href="https://en.wikipedia.org/wiki/Difference_quotient" target="_blank">\
         Wikipedia article about difference quotient</a></div>'
         )

    x = SR.var('x')
    f = symbolic_expression(f).function(x)
    fmax = f.find_local_maximum(interval[0], interval[1])[0]
    fmin = f.find_local_minimum(interval[0], interval[1])[0]
    f_height = fmax - fmin
    measure_y = fmin - 0.1*f_height

    measure_0 = line2d([(x0, measure_y), (a, measure_y)], rgbcolor='black')
    measure_1 = line2d([(x0, measure_y + 0.02*f_height), (x0, measure_y-0.02*f_height)], rgbcolor='black')
    measure_2 = line2d([(a, measure_y + 0.02*f_height), (a, measure_y-0.02*f_height)], rgbcolor='black')
    text_x0 = text("x0", (x0, measure_y - 0.05*f_height), rgbcolor='black')
    text_a = text("a", (a, measure_y - 0.05*f_height), rgbcolor='black')
    measure = measure_0 + measure_1 + measure_2 + text_x0 + text_a

    tanf = symbolic_expression((f(x0)-f(a))*(x-a)/(x0-a)+f(a)).function(x)

    fplot = plot(f(x), x, interval[0], interval[1])
    tanplot = plot(tanf(x), x, interval[0], interval[1], rgbcolor='#FF0000')
    points = point([(x0, f(x0)), (a, f(a))], pointsize=20, rgbcolor='#005500')
    dashline = line2d([(x0, f(x0)), (x0, f(a)), (a, f(a))], rgbcolor='#005500', linestyle='--')
    html('<h2>Difference Quotient</h2>')
    show(fplot + tanplot + points + dashline + measure, xmin=interval[0], xmax=interval[1], ymin=fmin-0.2*f_height, ymax=fmax)
    html(r"<br>$\text{Line's equation:}$")
    html(r"$y = %s$<br>" % tanf(x))
    html(r"$\text{Slope:}$")
    html(r"$k = \frac{f(x_0)-f(a)}{x_0-a} = %s$<br>" % (N(derivative(tanf(x), x), digits=5)))


@library_interact(
    A=lambda: slider(-7, 7, 1, 1),
    B=lambda: slider(-7, 7, 1, 1),
    C=lambda: slider(-7, 7, 1, -2),
)
def quadratic_equation(A, B, C):
    r"""
    This is a demo interact for solving quadratic equations based on work by
    Lauri Ruotsalainen, 2010.

    INPUT:

    - ``A`` -- integer slider
    - ``B`` -- integer slider
    - ``C`` -- integer slider

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.quadratic_equation()
        ...Interactive function <function quadratic_equation at ...> with 3 widgets
          A: IntSlider(value=1, description='A', max=7, min=-7)
          B: IntSlider(value=1, description='B', max=7, min=-7)
          C: IntSlider(value=-2, description='C', max=7, min=-7)
    """
    x = SR.var('x')
    f = symbolic_expression(A*x**2 + B*x + C).function(x)
    html('<h2>The Solutions of the Quadratic Equation</h2>')
    html("$%s = 0$" % f(x))

    show(plot(f(x), x, (-10, 10), ymin=-10, ymax=10), aspect_ratio=1, figsize=4)

    d = B**2 - 4*A*C

    if d < 0:
        color = "Red"
        sol = r"\text{solution} \in \mathbb{C}"
    elif d == 0:
        color = "Blue"
        sol = -B/(2*A)
    else:
        color = "Green"
        a = (-B+sqrt(B**2-4*A*C))/(2*A)
        b = (-B-sqrt(B**2-4*A*C))/(2*A)
        sol = r"\begin{cases}%s\\%s\end{cases}" % (latex(a), latex(b))

    if B < 0:
        dis1 = "(%s)^2-4*%s*%s" % (B, A, C)
    else:
        dis1 = "%s^2-4*%s*%s" % (B, A, C)
    dis2 = r"\color{%s}{%s}" % (color, d)

    html("$Ax^2 + Bx + C = 0$")
    calc = r"$x = \frac{-B\pm\sqrt{B^2-4AC}}{2A} = " + \
           r"\frac{-%s\pm\sqrt{%s}}{2*%s} = " + \
           r"\frac{-%s\pm\sqrt{%s}}{%s} = %s$"
    html(calc % (B, dis1, A, B, dis2, (2*A), sol))


@library_interact(
    a0=lambda: slider(0, 360, 1, 30, label='A'),
    a1=lambda: slider(0, 360, 1, 180, label='B'),
    a2=lambda: slider(0, 360, 1, 300, label='C'),
)
def trigonometric_properties_triangle(a0, a1, a2):
    r"""
    This is an interact for demonstrating trigonometric properties
    in a triangle based on work by Lauri Ruotsalainen, 2010.

    INPUT:

    - ``a0`` -- angle
    - ``a1`` -- angle
    - ``a2`` -- angle

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.geometry.trigonometric_properties_triangle()
        ...Interactive function <function trigonometric_properties_triangle at ...> with 3 widgets
          a0: IntSlider(value=30, description='A', max=360)
          a1: IntSlider(value=180, description='B', max=360)
          a2: IntSlider(value=300, description='C', max=360)
    """
    import math

    # Returns the distance between points (x1,y1) and (x2,y2)
    def distance(x1_y1, x2_y2):
        (x1, y1) = x1_y1
        (x2, y2) = x2_y2
        return sqrt((x2-x1)**2 + (y2-y1)**2)

    # Returns an angle (in radians) when sides a and b
    # are adjacent and the side c is opposite to the angle
    def angle(a, b, c):
        a,b,c = map(float,[a,b,c])
        return acos(0.5 * (b**2 + c**2 - a**2) / (b * c))

    # Returns the area of a triangle when an angle alpha
    # and adjacent sides a and b are known
    def area(alpha, a, b):
        return 0.5 * a * b * sin(alpha)

    xy = [0]*3
    html('<h2>Trigonometric Properties of a Triangle</h2>')
    # Coordinates of the angles
    a = [math.radians(float(x)) for x in [a0, a1, a2]]
    for i in range(3):
        xy[i] = (cos(a[i]), sin(a[i]))

    # Side lengths (bc, ca, ab) corresponding to triangle vertices (a, b, c)
    al = [distance(xy[1], xy[2]), distance(xy[2], xy[0]), distance(xy[0], xy[1])]

    # The angles (a, b, c) in radians
    ak = [angle(al[0], al[1], al[2]), angle(al[1], al[2], al[0]), angle(al[2], al[0], al[1])]

    # The area of the triangle
    A = area(ak[0], al[1], al[2])

    unit_circle = circle((0, 0), 1, aspect_ratio=1)

    # Triangle
    triangle = line([xy[0], xy[1], xy[2], xy[0]], rgbcolor='black')
    triangle_points = point(xy, pointsize=30)

    # Labels of the angles drawn in a distance from points
    a_label = text("A", (xy[0][0]*1.07, xy[0][1]*1.07))
    b_label = text("B", (xy[1][0]*1.07, xy[1][1]*1.07))
    c_label = text("C", (xy[2][0]*1.07, xy[2][1]*1.07))
    labels = a_label + b_label + c_label

    show(unit_circle + triangle + triangle_points + labels, figsize=[5, 5], xmin=-1, xmax=1, ymin=-1, ymax=1)
    html(r"$\angle A = {%.3f}^{\circ},$ $\angle B = {%.3f}^{\circ},$ $\angle C = {%.3f}^{\circ}$"
         % (math.degrees(ak[0]), math.degrees(ak[1]), math.degrees(ak[2])))
    html(r"$AB = %.6f$, $BC = %.6f$, $CA = %.6f$" % (al[2], al[0], al[1]))
    html(r"Area of triangle $ABC = %.6f$" % A)


@library_interact(
    function=lambda: selector([(0, sin(x)), (1, cos(x)), (2, tan(x))]),
    x=lambda: slider(0, 2 * pi, 0.005 * pi, 0),
)
def unit_circle(function, x):
    r"""
    This is an interact for Sin, Cos and Tan in the Unit Circle
    based on work by Lauri Ruotsalainen, 2010.

    INPUT:

    - ``function`` -- select Sin, Cos or Tan
    - ``x`` -- slider to select angle in unit circle

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.geometry.unit_circle()
        ...Interactive function <function unit_circle at ...> with 2 widgets
          function: Dropdown(description='function', options=(('sin(x)', 0), ('cos(x)', 1), ('tan(x)', 2)), value=0)
          x: TransformFloatSlider(value=0.0, description='x', max=6.283185307179586, step=0.015707963267948967)
    """
    xy = (cos(x), sin(x))
    t = SR.var('t')
    html('<div style="white-space: normal;">Lines of the same color have\
         the same length</div>')

    # Unit Circle
    C = circle((0, 0), 1, figsize=[5, 5], aspect_ratio=1)
    C_line = line([(0, 0), (xy[0], xy[1])], rgbcolor='black')
    C_point = point((xy[0], xy[1]), pointsize=40, rgbcolor='green')
    C_inner = parametric_plot((cos(t), sin(t)), (t, 0, x + 0.001), color='green', thickness=3)
    C_outer = parametric_plot((0.1 * cos(t), 0.1 * sin(t)), (t, 0, x + 0.001), color='black')
    C_graph = C + C_line + C_point + C_inner + C_outer

    # Graphics related to the graph of the function
    G_line = line([(0, 0), (x, 0)], rgbcolor='green', thickness=3)
    G_point = point((x, 0), pointsize=30, rgbcolor='green')
    G_graph = G_line + G_point

    # Sine
    if function == 0:
        Gf = plot(sin(t), t, 0, 2*pi, axes_labels=("x", "sin(x)"))
        Gf_point = point((x, sin(x)), pointsize=30, rgbcolor='red')
        Gf_line = line([(x, 0),(x, sin(x))], rgbcolor='red')
        Cf_point = point((0, xy[1]), pointsize=40, rgbcolor='red')
        Cf_line1 = line([(0, 0), (0, xy[1])], rgbcolor='red', thickness=3)
        Cf_line2 = line([(0, xy[1]), (xy[0], xy[1])], rgbcolor='purple', linestyle='--')
    # Cosine
    elif function == 1:
        Gf = plot(cos(t), t, 0, 2*pi, axes_labels=("x", "cos(x)"))
        Gf_point = point((x, cos(x)), pointsize=30, rgbcolor='red')
        Gf_line = line([(x, 0), (x, cos(x))], rgbcolor='red')
        Cf_point = point((xy[0], 0), pointsize=40, rgbcolor='red')
        Cf_line1 = line([(0, 0), (xy[0], 0)], rgbcolor='red', thickness=3)
        Cf_line2 = line([(xy[0], 0), (xy[0], xy[1])], rgbcolor='purple', linestyle='--')
    # Tangent
    else:
        Gf = plot(tan(t), t, 0, 2*pi, ymin=-8, ymax=8, axes_labels=("x", "tan(x)"))
        Gf_point = point((x, tan(x)), pointsize=30, rgbcolor='red')
        Gf_line = line([(x, 0), (x, tan(x))], rgbcolor='red')
        Cf_point = point((1, tan(x)), pointsize=40, rgbcolor='red')
        Cf_line1 = line([(1, 0), (1, tan(x))], rgbcolor='red', thickness=3)
        Cf_line2 = line([(xy[0], xy[1]), (1, tan(x))], rgbcolor='purple', linestyle='--')

    C_graph += Cf_point + Cf_line1 + Cf_line2
    G_graph += Gf + Gf_point + Gf_line

    show(graphics_array([C_graph, G_graph]))


@library_interact(
    title=lambda: text_control("<h2>Special points in triangle</h2>"),
    a0=lambda: slider(0, 360, 1, 30, label='A'),
    a1=lambda: slider(0, 360, 1, 180, label='B'),
    a2=lambda: slider(0, 360, 1, 300, label='C'),
    show_median=lambda: checkbox(False, label='Medians'),
    show_pb=lambda: checkbox(False, label="Perpendicular Bisectors"),
    show_alt=lambda: checkbox(False, label='Altitudes'),
    show_ab=lambda: checkbox(False, label="Angle Bisectors"),
    show_incircle=lambda: checkbox(False, label='Incircle'),
    show_euler=lambda: checkbox(False, label="Euler's Line"),
)
def special_points(
    title,
    a0,
    a1,
    a2,
    show_median,
    show_pb,
    show_alt,
    show_ab,
    show_incircle,
    show_euler,
):
    r"""
    This interact demo shows special points in a triangle
    based on work by Lauri Ruotsalainen, 2010.

    INPUT:

    - ``a0`` -- angle
    - ``a1`` -- angle
    - ``a2`` -- angle
    - ``show_median`` -- checkbox
    - ``show_pb`` -- checkbox to show perpendicular bisectors
    - ``show_alt`` -- checkbox to show altitudes
    - ``show_ab`` -- checkbox to show angle bisectors
    - ``show_incircle`` -- checkbox to show incircle
    - ``show_euler`` -- checkbox to show euler's line

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.geometry.special_points()
        ...Interactive function <function special_points at ...> with 10 widgets
          title: HTMLText(value='<h2>Special points in triangle</h2>')
          a0: IntSlider(value=30, description='A', max=360)
          a1: IntSlider(value=180, description='B', max=360)
          a2: IntSlider(value=300, description='C', max=360)
          show_median: Checkbox(value=False, description='Medians')
          show_pb: Checkbox(value=False, description='Perpendicular Bisectors')
          show_alt: Checkbox(value=False, description='Altitudes')
          show_ab: Checkbox(value=False, description='Angle Bisectors')
          show_incircle: Checkbox(value=False, description='Incircle')
          show_euler: Checkbox(value=False, description="Euler's Line")
    """
    import math
    # Return the intersection point of the bisector of the angle <(A[a],A[c],A[b]) and the unit circle. Angles given in radians.

    def half(A, a, b, c):
        if (A[a] < A[b] and (A[c] < A[a] or A[c] > A[b])) or (A[a] > A[b] and (A[c] > A[a] or A[c] < A[b])):
            p = A[a] + 0.5 * (A[b] - A[a])
        else:
            p = A[b] + 0.5 * (2*pi - (A[b]-A[a]))
        return (math.cos(p), math.sin(p))

    # Returns the distance between points (x1,y1) and (x2,y2)
    def distance(x1_y1, x2_y2):
        (x1, y1) = x1_y1
        (x2, y2) = x2_y2
        return math.sqrt((x2-x1)**2 + (y2-y1)**2)

    # Returns the line (graph) going through points (x1,y1) and (x2,y2)
    def line_to_points(x1_y1, x2_y2, **plot_kwargs):
        (x1, y1) = x1_y1
        (x2, y2) = x2_y2
        return plot((y2-y1) / (x2-x1) * (x-x1) + y1, (x,-3,3), **plot_kwargs)

    # Coordinates of the angles
    a = [math.radians(float(x)) for x in [a0, a1, a2]]
    xy = [(math.cos(a[i]), math.sin(a[i])) for i in range(3)]

    # Labels of the angles drawn in a distance from points
    a_label = text("A", (xy[0][0]*1.07, xy[0][1]*1.07))
    b_label = text("B", (xy[1][0]*1.07, xy[1][1]*1.07))
    c_label = text("C", (xy[2][0]*1.07, xy[2][1]*1.07))
    labels = a_label + b_label + c_label

    C = circle((0, 0), 1, aspect_ratio=1)

    # Triangle
    triangle = line([xy[0], xy[1], xy[2], xy[0]], rgbcolor='black')
    triangle_points = point(xy, pointsize=30)

    # Side lengths (bc, ca, ab) corresponding to triangle vertices (a, b, c)
    ad = [distance(xy[1], xy[2]), distance(xy[2], xy[0]), distance(xy[0], xy[1])]

    # Midpoints of edges (bc, ca, ab)
    a_middle = [
        (0.5 * (xy[1][0] + xy[2][0]), 0.5 * (xy[1][1] + xy[2][1])),
        (0.5 * (xy[2][0] + xy[0][0]), 0.5 * (xy[2][1] + xy[0][1])),
        (0.5 * (xy[0][0] + xy[1][0]), 0.5 * (xy[0][1] + xy[1][1]))
    ]

    # Incircle
    perimeter = float(ad[0] + ad[1] + ad[2])
    incircle_center = (
        (ad[0]*xy[0][0] + ad[1]*xy[1][0] + ad[2]*xy[2][0]) / perimeter,
        (ad[0]*xy[0][1] + ad[1]*xy[1][1] + ad[2]*xy[2][1]) / perimeter
    )

    if show_incircle:
        s = 0.5 * perimeter
        incircle_r = math.sqrt((s - ad[0]) * (s - ad[1]) * (s - ad[2]) / s)
        incircle_graph = circle(incircle_center, incircle_r) + point(incircle_center)
    else:
        incircle_graph = Graphics()

    # Angle Bisectors
    if show_ab:
        a_ab = line([xy[0], half(a, 1, 2, 0)], rgbcolor='blue', alpha=0.6)
        b_ab = line([xy[1], half(a, 2, 0, 1)], rgbcolor='blue', alpha=0.6)
        c_ab = line([xy[2], half(a, 0, 1, 2)], rgbcolor='blue', alpha=0.6)
        ab_point = point(incircle_center, rgbcolor='blue', pointsize=28)
        ab_graph = a_ab + b_ab + c_ab + ab_point
    else:
        ab_graph = Graphics()

    # Medians
    if show_median:
        a_median = line([xy[0], a_middle[0]], rgbcolor='green', alpha=0.6)
        b_median = line([xy[1], a_middle[1]], rgbcolor='green', alpha=0.6)
        c_median = line([xy[2], a_middle[2]], rgbcolor='green', alpha=0.6)
        median_point = point(
            (
                (xy[0][0]+xy[1][0]+xy[2][0])/3.0,
                (xy[0][1]+xy[1][1]+xy[2][1])/3.0
            ), rgbcolor='green', pointsize=28)
        median_graph = a_median + b_median + c_median + median_point
    else:
        median_graph = Graphics()

    # Perpendicular Bisectors
    if show_pb:
        a_pb = line_to_points(a_middle[0], half(a, 1, 2, 0), rgbcolor='red', alpha=0.6)
        b_pb = line_to_points(a_middle[1], half(a, 2, 0, 1), rgbcolor='red', alpha=0.6)
        c_pb = line_to_points(a_middle[2], half(a, 0, 1, 2), rgbcolor='red', alpha=0.6)
        pb_point = point((0, 0), rgbcolor='red', pointsize=28)
        pb_graph = a_pb + b_pb + c_pb + pb_point
    else:
        pb_graph = Graphics()

    # Altitudes
    if show_alt:
        xA, xB, xC = xy[0][0], xy[1][0], xy[2][0]
        yA, yB, yC = xy[0][1], xy[1][1], xy[2][1]
        a_alt = plot(((xC-xB)*x+(xB-xC)*xA)/(yB-yC)+yA, (x,-3,3), rgbcolor='brown', alpha=0.6)
        b_alt = plot(((xA-xC)*x+(xC-xA)*xB)/(yC-yA)+yB, (x,-3,3), rgbcolor='brown', alpha=0.6)
        c_alt = plot(((xB-xA)*x+(xA-xB)*xC)/(yA-yB)+yC, (x,-3,3), rgbcolor='brown', alpha=0.6)
        alt_lx = (xA*xB*(yA-yB)+xB*xC*(yB-yC)+xC*xA*(yC-yA)-(yA-yB)*(yB-yC)*(yC-yA))/(xC*yB-xB*yC+xA*yC-xC*yA+xB*yA-xA*yB)
        alt_ly = (yA*yB*(xA-xB)+yB*yC*(xB-xC)+yC*yA*(xC-xA)-(xA-xB)*(xB-xC)*(xC-xA))/(yC*xB-yB*xC+yA*xC-yC*xA+yB*xA-yA*xB)
        alt_intersection = point((alt_lx, alt_ly), rgbcolor='brown', pointsize=28)
        alt_graph = a_alt + b_alt + c_alt + alt_intersection
    else:
        alt_graph = Graphics()

    # Euler's Line
    if show_euler:
        euler_graph = line_to_points(
            (0, 0),
            (
                (xy[0][0]+xy[1][0]+xy[2][0])/3.0,
                (xy[0][1]+xy[1][1]+xy[2][1])/3.0
            ),
            rgbcolor='purple',
            thickness=2,
            alpha=0.7
        )
    else:
        euler_graph = Graphics()

    show(
        C + triangle + triangle_points + labels + ab_graph + median_graph +
        pb_graph + alt_graph + incircle_graph + euler_graph,
        figsize=[5,5], xmin=-1, xmax=1, ymin=-1, ymax=1
    )


@library_interact(
    n=lambda: slider(2, 10000, 100, default=1000, label="Number of Tosses"),
    interval=lambda: range_slider(
        0.0, 1.0, default=(0.45, 0.55), label="Plotting range (y)"
    ),
)
def coin(n, interval):
    r"""
    This interact demo simulates repeated tosses of a coin,
    based on work by Lauri Ruotsalainen, 2010.

    The points give the cumulative percentage of tosses which
    are heads in a given run of the simulation, so that the
    point `(x,y)` gives the percentage of the first `x` tosses
    that were heads; this proportion should approach .5, of
    course, if we are simulating a fair coin.

    INPUT:

    - ``n`` -- number of tosses
    - ``interval`` -- plot range along vertical axis

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.statistics.coin()
        ...Interactive function <function coin at ...> with 2 widgets
          n: IntSlider(value=1000, description='Number of Tosses', max=10000, min=2, step=100)
          interval: FloatRangeSlider(value=(0.45, 0.55), description='Plotting range (y)', max=1.0)
    """
    from random import random
    c = []
    k = 0.0
    for i in range(1, n + 1):
        k += random()
        c.append((i, k/i))
    show(point(c[1:], gridlines=[None, [0.5]], pointsize=1), ymin=interval[0], ymax=interval[1])


@library_interact(
    title=lambda: text_control("<h2>Bisection method</h2>"),
    f=lambda: input_box("x^2-2", label="f(x)"),
    interval=lambda: range_slider(-5, 5, default=(0, 4), label='range'),
    d=lambda: slider(1, 8, 1, 3, label="$10^{-d}$ precision"),
    maxn=lambda: slider(0, 50, 1, 10, label="max iterations"),
)
def bisection_method(title, f, interval, d, maxn):
    r"""
    Interact explaining the bisection method, based on similar interact
    explaining secant method and Wiliam Stein's example from wiki.

    INPUT:

    - ``f`` -- function
    - ``interval`` -- range slider for the search interval
    - ``d`` -- slider for the precision (`10^{-d}`)
    - ``maxn`` -- max number of iterations

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.secant_method()
        ...Interactive function <function secant_method at ...> with 5 widgets
          title: HTMLText(value='<h2>Secant method for numerical root finding</h2>')
          f: EvalText(value='x^2-2', description='f(x)', layout=Layout(max_width='81em'))
          interval: IntRangeSlider(value=(0, 4), description='range', max=5, min=-5)
          d: IntSlider(value=3, description='10^-d precision', max=16, min=1)
          maxn: IntSlider(value=10, description='max iterations', max=15)
    """
    def _bisection_method(f, a, b, maxn, eps):
        intervals = [(a, b)]
        round = 1
        two = float(2)
        while True:
            c = (b+a)/two
            if abs(f(c)) < h or round >= maxn:
                break
            fa = f(a)
            fb = f(b)
            fc = f(c)
            if abs(fc) < eps:
                return c, intervals
            if fa*fc < 0:
                b = c
            elif fc*fb < 0:
                a = c
            else:
                raise ValueError("f must have a sign change in the interval (%s,%s)" % (a,b))
            intervals.append((a, b))
            round += 1
        return c, intervals

    x = SR.var('x')
    f = symbolic_expression(f).function(x)
    a, b = interval
    h = 10**(-d)
    try:
        c, intervals = _bisection_method(f, float(a), float(b), maxn, h)
    except ValueError:
        print("f must have opposite sign at the endpoints of the interval")
        show(plot(f, a, b, color='red'), xmin=a, xmax=b)
    else:
        html(r"$\text{Precision }h = 10^{-d}=10^{-%s}=%.5f$" % (d, float(h)))
        html(r"${c = }%s$" % latex(c))
        html(r"${f(c) = }%s" % latex(f(c)))
        html(r"$%s \text{ iterations}" % len(intervals))
        P = plot(f, a, b, color='red')
        k = (P.ymax() - P.ymin()) / (1.5*len(intervals))
        L = sum(line([(c,k*i), (d,k*i)]) for i, (c,d) in enumerate(intervals) )
        L += sum(line([(c,k*i-k/4), (c,k*i+k/4)]) for i, (c,d) in enumerate(intervals) )
        L += sum(line([(d,k*i-k/4), (d,k*i+k/4)]) for i, (c,d) in enumerate(intervals) )
        show(P + L, xmin=a, xmax=b)


@library_interact(
    title=lambda: text_control("<h2>Secant method for numerical root finding</h2>"),
    f=lambda: input_box("x^2-2", label="f(x)"),
    interval=lambda: range_slider(-5, 5, default=(0, 4), label='range'),
    d=lambda: slider(1, 16, 1, 3, label="10^-d precision"),
    maxn=lambda: slider(0, 15, 1, 10, label="max iterations"),
)
def secant_method(title, f, interval, d, maxn):
    r"""
    Interact explaining the secant method, based on work by
    Lauri Ruotsalainen, 2010.
    Originally this is based on work by William Stein.

    INPUT:

    - ``f`` -- function
    - ``interval`` -- range slider for the search interval
    - ``d`` -- slider for the precision (10^-d)
    - ``maxn`` -- max number of iterations

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.secant_method()
        ...Interactive function <function secant_method at ...> with 5 widgets
          title: HTMLText(value='<h2>Secant method for numerical root finding</h2>')
          f: EvalText(value='x^2-2', description='f(x)', layout=Layout(max_width='81em'))
          interval: IntRangeSlider(value=(0, 4), description='range', max=5, min=-5)
          d: IntSlider(value=3, description='10^-d precision', max=16, min=1)
          maxn: IntSlider(value=10, description='max iterations', max=15)
    """
    def _secant_method(f, a, b, maxn, h):
        intervals = [(a,b)]
        round = 1
        while True:
            c = b-(b-a)*f(b)/(f(b)-f(a))
            if abs(f(c)) < h or round >= maxn:
                break
            a, b = b, c
            intervals.append((a,b))
            round += 1
        return c, intervals

    x = SR.var('x')
    f = symbolic_expression(f).function(x)
    a, b = interval
    h = 10**(-d)
    if float(f(a)*f(b)) > 0:
        print("f must have opposite sign at the endpoints of the interval")
        show(plot(f, a, b, color='red'), xmin=a, xmax=b)
    else:
        c, intervals = _secant_method(f, float(a), float(b), maxn, h)
        html(r"$\text{Precision }h = 10^{-d}=10^{-%s}=%.5f$" % (d, float(h)))
        html(r"${c = }%s$" % latex(c))
        html(r"${f(c) = }%s" % latex(f(c)))
        html(r"$%s \text{ iterations}" % len(intervals))
        P = plot(f, a, b, color='red')
        k = (P.ymax() - P.ymin()) / (1.5*len(intervals))
        L = sum(line([(c,k*i), (d,k*i)]) for i, (c,d) in enumerate(intervals) )
        L += sum(line([(c,k*i-k/4), (c,k*i+k/4)]) for i, (c,d) in enumerate(intervals) )
        L += sum(line([(d,k*i-k/4), (d,k*i+k/4)]) for i, (c,d) in enumerate(intervals) )
        S = sum(line([(c,f(c)), (d,f(d)), (d-(d-c)*f(d)/(f(d)-f(c)), 0)], color='green') for (c, d) in intervals)
        show(P + L + S, xmin=a, xmax=b)


@library_interact(
    title=lambda: text_control("<h2>Newton method</h2>"),
    f=lambda: input_box("x^2 - 2"),
    c=lambda: slider(-10, 10, default=6, label="Start ($x$)"),
    d=lambda: slider(1, 16, 1, 3, label="$10^{-d}$ precision"),
    maxn=lambda: slider(0, 15, 1, 10, label="max iterations"),
    interval=lambda: range_slider(-10, 10, default=(0, 6), label='Interval'),
    list_steps=lambda: checkbox(default=False, label="List steps"),
)
def newton_method(title, f, c, d, maxn, interval, list_steps):
    r"""
    Interact explaining the Newton method, based on work by
    Lauri Ruotsalainen, 2010.
    Originally this is based on work by William Stein.

    INPUT:

    - ``f`` -- function
    - ``c`` -- starting position (`x`)
    - ``d`` -- slider for the precision (`10^{-d}`)
    - ``maxn`` -- max number of iterations
    - ``interval`` -- range slider for the search interval
    - ``list_steps`` -- checkbox, if ``True`` shows the steps numerically

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.newton_method()
        ...Interactive function <function newton_method at ...> with 7 widgets
          title: HTMLText(value='<h2>Newton method</h2>')
          f: EvalText(value='x^2 - 2', description='f', layout=Layout(max_width='81em'))
          c: IntSlider(value=6, description='Start ($x$)', max=10, min=-10)
          d: IntSlider(value=3, description='$10^{-d}$ precision', max=16, min=1)
          maxn: IntSlider(value=10, description='max iterations', max=15)
          interval: IntRangeSlider(value=(0, 6), description='Interval', max=10, min=-10)
          list_steps: Checkbox(value=False, description='List steps')
    """
    def _newton_method(f, c, maxn, h):
        midpoints = [c]
        round = 1
        while True:
            c = c-f(c)/f.derivative(x)(x=c)
            midpoints.append(c)
            if f(c-h)*f(c+h) < 0 or round == maxn:
                break
            round += 1
        return c, midpoints

    x = SR.var('x')
    f = symbolic_expression(f).function(x)
    a, b = interval
    h = 10**(-d)
    c, midpoints = _newton_method(f, float(c), maxn, 0.5 * h)
    html(r"$\text{Precision } 2h = %s$" % latex(float(h)))
    html(r"${c = }%s$" % c)
    html(r"${f(c) = }%s" % latex(f(c)))
    html(r"$%s \text{ iterations}" % len(midpoints))
    if list_steps:
        s = [["$n$", "$x_n$", "$f(x_n)$", r"$f(x_n-h)\,f(x_n+h)$"]]
        for i, c in enumerate(midpoints):
            s.append([i+1, c, f(c), (c-h)*f(c+h)])
        pretty_print(table(s, header_row=True))
    else:
        P = plot(f, x, interval, color='blue')
        L = sum(line([(c, 0), (c, f(c))], color='green') for c in midpoints[:-1])
        for i in range(len(midpoints) - 1):
            L += line([(midpoints[i], f(midpoints[i])), (midpoints[i+1], 0)], color='red')
        show(P + L, xmin=interval[0], xmax=interval[1], ymin=P.ymin(), ymax=P.ymax())


@library_interact(
    title=lambda: text_control("<h2>Trapezoid integration</h2>"),
    f=lambda: input_box(default="x^2-5*x + 10", label="$f(x)=$"),
    n=lambda: slider(1, 100, 1, 5, label="# divisions"),
    interval_input=lambda: selector(
        ["from slider", "from keyboard"],
        label="Integration interval",
        buttons=True,
    ),
    interval_s=lambda: range_slider(-10, 10, default=(0, 8), label="slider: "),
    interval_g=lambda: input_grid(1, 2, default=[[0, 8]], label="keyboard: "),
    output_form=lambda: selector(
        ["traditional", "table", "none"], label="Computations form", buttons=True
    ),
)
def trapezoid_integration(
    title, f, n, interval_input, interval_s, interval_g, output_form
):
    r"""
    Interact explaining the trapezoid method for definite integrals.

    Based on work by
    Lauri Ruotsalainen, 2010 (based on the application "Numerical integrals with various rules"
    by Marshall Hampton and Nick Alexander)

    INPUT:

    - ``f`` -- function of variable x to integrate
    - ``n`` -- number of divisions
    - ``interval_input`` -- switches the input for interval between slider and keyboard
    - ``interval_s`` -- slider for interval to integrate
    - ``interval_g`` -- input grid for interval to integrate
    - ``output_form`` -- the computation is formatted in a traditional form, in a table or missing

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.trapezoid_integration()
        ...Interactive function <function trapezoid_integration at ...> with 7 widgets
          title: HTMLText(value='<h2>Trapezoid integration</h2>')
          f: EvalText(value='x^2-5*x + 10', description='$f(x)=$', layout=Layout(max_width='81em'))
          n: IntSlider(value=5, description='# divisions', min=1)
          interval_input: ToggleButtons(description='Integration interval', options=('from slider', 'from keyboard'), value='from slider')
          interval_s: IntRangeSlider(value=(0, 8), description='slider: ', max=10, min=-10)
          interval_g: Grid(value=[[0, 8]], children=(Label(value='keyboard: '), VBox(children=(EvalText(value='0', layout=Layout(max_width='5em')),)), VBox(children=(EvalText(value='8', layout=Layout(max_width='5em')),))))
          output_form: ToggleButtons(description='Computations form', options=('traditional', 'table', 'none'), value='traditional')
    """
    xs = []
    ys = []
    if interval_input == 'from slider':
        interval = interval_s
    else:
        interval = interval_g[0]
    h = float(interval[1]-interval[0])/n
    x = SR.var('x')
    f = symbolic_expression(f).function(x)

    trapezoids = Graphics()

    for i in range(n):
        xi = interval[0] + i*h
        yi = f(xi)
        trapezoids += line([[xi, 0], [xi, yi], [xi + h, f(xi + h)],[xi + h, 0],[xi, 0]], rgbcolor=(1, 0, 0))
        xs.append(xi)
        ys.append(yi)
    xs.append(xi + h)
    ys.append(f(xi + h))

    html(r'Function $f(x)=%s$' % latex(f(x)))
    show(plot(f, interval[0], interval[1]) + trapezoids,
         xmin=interval[0], xmax=interval[1])

    numeric_value = integral_numerical(f, interval[0], interval[1])[0]
    approx = h * (ys[0]/2 + sum([ys[i] for i in range(1,n)]) + ys[n]/2)

    html(r'Integral value to seven decimal places is: $\displaystyle\int_{%.2f}^{%.2f} {f(x) \, \mathrm{d}x} = %.6f$' % (
            interval[0], interval[1], N(numeric_value, digits=7))
         )

    if output_form == 'traditional':
        sum_formula_html = r"\frac {d}{2} \cdot \left[f(x_0) + %s + f(x_{%s})\right]" % (
            ' + '.join([ "2 f(x_{%s})" % i for i in range(1,n)]),
            n
            )
        sum_placement_html = r"\frac{%.2f}{2} \cdot \left[f(%.2f) + %s + f(%.2f)\right]" % (
            h,
            N(xs[0], digits=5),
            ' + '.join([ "2 f(%.2f)" % N(i, digits=5) for i in xs[1:-1]]),
            N(xs[n], digits=5)
            )
        sum_values_html = r"\frac{%.2f}{2} \cdot \left[%.2f + %s + %.2f\right]" % (
            h,
            N(ys[0], digits=5),
            ' + '.join([ r"2\cdot %.2f" % N(i, digits=5) for i in ys[1:-1]]),
            N(ys[n], digits=5)
            )

        html(r'''
            <div class="math">
            \begin{align*}
            \int_{%.2f}^{%.2f} {f(x) \, \mathrm{d}x}
                & \approx %s \\
                & = %s \\
                & = %s \\
                & = %s
            \end{align*}
            </div>
        ''' % (
                interval[0], interval[1],
                sum_formula_html, sum_placement_html, sum_values_html,
                N(approx, digits=7)
        ))
    elif output_form == 'table':
        s = [['$i$', '$x_i$', '$f(x_i)$', '$m$', r'$m\cdot f(x_i)$']]
        for i in range(n+1):
            if i == 0 or i == n:
                j = 1
            else:
                j = 2
            s.append([i, xs[i], ys[i],j,N(j*ys[i])])
        pretty_print(table(s, header_row=True))


@library_interact(
    title=lambda: text_control("<h2>Simpson integration</h2>"),
    f=lambda: input_box(default='x*sin(x)+x+1', label="$f(x)=$"),
    n=lambda: slider(2, 100, 2, 6, label="# divisions"),
    interval_input=lambda: selector(
        ["from slider", "from keyboard"],
        label="Integration interval",
        buttons=True,
    ),
    interval_s=lambda: range_slider(-10, 10, default=(0, 10), label="slider: "),
    interval_g=lambda: input_grid(1, 2, default=[[0, 10]], label="keyboard: "),
    output_form=lambda: selector(
        ["traditional", "table", "none"], label="Computations form", buttons=True
    ),
)
def simpson_integration(
    title,
    f,
    n,
    interval_input,
    interval_s,
    interval_g,
    output_form,
):
    r"""
    Interact explaining the simpson method for definite integrals.

    Based on work by
    Lauri Ruotsalainen, 2010 (based on the application "Numerical integrals with various rules"
    by Marshall Hampton and Nick Alexander)

    INPUT:

    - ``f`` -- function of variable x to integrate
    - ``n`` -- number of divisions (mult. of 2)
    - ``interval_input`` -- switches the input for interval between slider and keyboard
    - ``interval_s`` -- slider for interval to integrate
    - ``interval_g`` -- input grid for interval to integrate
    - ``output_form`` -- the computation is formatted in a traditional form, in a table or missing

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.simpson_integration()
        ...Interactive function <function simpson_integration at ...> with 7 widgets
          title: HTMLText(value='<h2>Simpson integration</h2>')
          f: EvalText(value='x*sin(x)+x+1', description='$f(x)=$', layout=Layout(max_width='81em'))
          n: IntSlider(value=6, description='# divisions', min=2, step=2)
          interval_input: ToggleButtons(description='Integration interval', options=('from slider', 'from keyboard'), value='from slider')
          interval_s: IntRangeSlider(value=(0, 10), description='slider: ', max=10, min=-10)
          interval_g: Grid(value=[[0, 10]], children=(Label(value='keyboard: '), VBox(children=(EvalText(value='0', layout=Layout(max_width='5em')),)), VBox(children=(EvalText(value='10', layout=Layout(max_width='5em')),))))
          output_form: ToggleButtons(description='Computations form', options=('traditional', 'table', 'none'), value='traditional')
    """
    x = SR.var("x")
    f = symbolic_expression(f).function(x)
    if interval_input == 'from slider':
        interval = interval_s
    else:
        interval = interval_g[0]

    def parabola(a, b, c):
        from sage.symbolic.relation import solve
        A, B, C = SR.var("A, B, C")
        K = solve([A*a[0]**2+B*a[0]+C == a[1], A*b[0]**2+B*b[0]+C == b[1], A*c[0]**2+B*c[0]+C == c[1]], [A, B, C], solution_dict=True)[0]
        f = K[A]*x**2+K[B]*x+K[C]
        return f
    xs = []
    ys = []
    dx = float(interval[1]-interval[0])/n

    for i in range(n+1):
        xs.append(interval[0] + i*dx)
        ys.append(f(x=xs[-1]))

    parabolas = Graphics()
    lines = Graphics()

    for i in range(0, n-1, 2):
        p = parabola((xs[i],ys[i]),(xs[i+1],ys[i+1]),(xs[i+2],ys[i+2]))
        parabolas += plot(p(x=x), (x, xs[i], xs[i+2]), color='red')
        lines += line([(xs[i],ys[i]), (xs[i],0), (xs[i+2],0)],color='red')
        lines += line([(xs[i+1],ys[i+1]), (xs[i+1],0)], linestyle='-.', color='red')

    lines += line([(xs[-1],ys[-1]), (xs[-1],0)], color='red')

    html(r'Function $f(x)=%s$' % latex(f(x)))

    show(plot(f(x), x, interval[0], interval[1]) + parabolas + lines,
         xmin=interval[0], xmax=interval[1])

    numeric_value = integral_numerical(f,interval[0],interval[1])[0]
    approx = dx/3 * (ys[0] + sum([4*ys[i] for i in range(1,n,2)]) + sum([2*ys[i] for i in range(2,n,2)]) + ys[n])

    html(r'Integral value to seven decimal places is: $\displaystyle\int_{%.2f}^{%.2f} {f(x) \, \mathrm{d}x} = %.6f$' %
         (interval[0],interval[1],
          N(numeric_value,digits=7)))

    if output_form == 'traditional':
        sum_formula_html = r"\frac{d}{3} \cdot \left[ f(x_0) + %s + f(x_{%s})\right]" % (
            ' + '.join(r"%s \cdot f(x_{%s})" % (i % 2 * (-2) + 4, i + 1)
                       for i in range(n-1)),
            n
            )

        sum_placement_html = r"\frac{%.2f}{3} \cdot \left[ f(%.2f) +  %s + f(%.2f)\right]" % (
            dx,
            N(xs[0],digits=5),
            ' + '.join(r"%s \cdot f(%.2f)" % (i % 2 * (-2) + 4, N(xk, digits=5))
                       for i, xk in enumerate(xs[1:-1])),
            N(xs[n],digits=5)
            )

        sum_values_html = r"\frac{%.2f}{3} \cdot \left[ %s %s %s\right]" % (
            dx,
            "%.2f + " % N(ys[0],digits=5),
            ' + '.join(r"%s \cdot %.2f" % (i % 2 * (-2) + 4, N(yk, digits=5))
                       for i, yk in enumerate(ys[1:-1])),
            " + %.2f" % N(ys[n],digits=5)
            )

        html(r'''
        <div class="math">
        \begin{align*}
        \int_{%.2f}^{%.2f} {f(x) \, \mathrm{d}x}
            & \approx %s \\
            & = %s \\
            & = %s \\
            & = %.6f
        \end{align*}
        </div>
        ''' % (
                interval[0], interval[1],
                sum_formula_html, sum_placement_html, sum_values_html,
                N(approx,digits=7)
                ))
    elif output_form == 'table':
        s = [['$i$', '$x_i$', '$f(x_i)$', '$m$', r'$m\cdot f(x_i)$']]
        for i in range(n + 1):
            if i == 0 or i == n:
                j = 1
            else:
                j = (i + 1) % 2 * (-2) + 4
            s.append([i, xs[i], ys[i], j, N(j*ys[i])])
        s.append(['', '', '', r'$\sum$', '$%s$' % latex(3/dx*approx)])
        pretty_print(table(s, header_row=True))
        html(r'$\int_{%.2f}^{%.2f} {f(x) \, \mathrm{d}x}\approx\frac {%.2f}{3}\cdot %s=%s$' %
             (interval[0], interval[1],dx,latex(3/dx*approx),latex(approx)))


@library_interact(
    title=lambda: text_control("<h2>Riemann integral with random sampling</h2>"),
    f=lambda: input_box("x^2+1", label='$f(x)=$', width=40),
    n=lambda: slider(1, 30, 1, 5, label="# divisions"),
    hr1=lambda: text_control("<hr>"),
    interval_input=lambda: selector(
        ["from slider", "from keyboard"],
        label="Integration interval",
        buttons=True,
    ),
    interval_s=lambda: range_slider(-5, 10, default=(0, 2), label="slider: "),
    interval_g=lambda: input_grid(1, 2, default=[[0, 2]], label="keyboard: "),
    hr2=lambda: text_control("<hr>"),
    list_table=lambda: checkbox(default=False, label="List table"),
    auto_update=lambda: False,
)
def riemann_sum(
    title,
    f,
    n,
    hr1,
    interval_input,
    interval_s,
    interval_g,
    hr2,
    list_table,
    auto_update=False,
):
    r"""
    Interact explaining the definition of Riemann integral.

    INPUT:

    - ``f`` -- function of variable x to integrate
    - ``n`` -- number of divisions
      - ``interval_input`` -- switches the input for interval between slider and keyboard
      - ``interval_s`` -- slider for interval to integrate
      - ``interval_g`` -- input grid for interval to integrate
    - ``list_table`` -- print table with values of the function

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.riemann_sum()
        Manual interactive function <function riemann_sum at ...> with 9 widgets
          title: HTMLText(value='<h2>Riemann integral with random sampling</h2>')
          f: EvalText(value='x^2+1',... description='$f(x)=$', layout=Layout(max_width='41em'))
          n: IntSlider(value=5, description='# divisions', max=30, min=1)
          hr1: HTMLText(value='<hr>')
          interval_input: ToggleButtons(description='Integration interval', options=('from slider', 'from keyboard'), value='from slider')
          interval_s: IntRangeSlider(value=(0, 2), description='slider: ', max=10, min=-5)
          interval_g: Grid(value=[[0, 2]], children=(Label(value='keyboard: '), VBox(children=(EvalText(value='0', layout=Layout(max_width='5em')),)), VBox(children=(EvalText(value='2', layout=Layout(max_width='5em')),))))
          hr2: HTMLText(value='<hr>')
          list_table: Checkbox(value=False, description='List table')

    AUTHORS:

    - Robert Marik (2010-08)
    """
    x = SR.var('x')
    from random import random
    if interval_input == 'from slider':
        a = interval_s[0]
        b = interval_s[1]
    else:
        a = interval_g[0][0]
        b = interval_g[0][1]
    func = symbolic_expression(f).function(x)
    division = [a]+[a+random()*(b-a) for i in range(n-1)]+[b]
    division = sorted(division)
    xs = [division[i]+random()*(division[i+1]-division[i]) for i in range(n)]
    ys = [func(x_val) for x_val in xs]
    rects = Graphics()
    for i in range(n):
        body = [[division[i], 0], [division[i], ys[i]],
                [division[i+1], ys[i]], [division[i+1], 0]]
        if ys[i].n() > 0:
            color_rect = 'green'
        else:
            color_rect = 'red'
        rects = rects + polygon2d(body, rgbcolor=color_rect, alpha=0.1)\
         + point((xs[i], ys[i]), rgbcolor=(1, 0, 0))\
         + line(body, rgbcolor='black', zorder=-1)
    html('<small>Adjust your data and click Update button. Click repeatedly for another random values.</small>')

    show(plot(func(x),(x,a,b),zorder=5) + rects)
    delka_intervalu = [division[i+1]-division[i] for i in range(n)]
    if list_table:
        pretty_print(table([
            ["$i$", "$[x_{i-1},x_i]$", r"$\eta_i$", r"$f(\eta_i)$", "$x_{i}-x_{i-1}$"]
        ] + [
            [i+1,[division[i],division[i+1]],xs[i],ys[i],delka_intervalu[i]] for i in range(n)
        ], header_row=True))

    html(r'Riemann sum: $\displaystyle\sum_{i=1}^{%s} f(\eta_i)(x_i-x_{i-1})=%s$ ' %
         (latex(n),latex(sum([ys[i]*delka_intervalu[i] for i in range(n)]))))
    html(r'Exact value of the integral $\displaystyle\int_{%s}^{%s}%s\,\mathrm{d}x=%s$' %
         (latex(a),latex(b),latex(func(x)),latex(integral_numerical(func(x),a,b)[0])))


x = SR.var("x")


@library_interact(
    f=lambda: sin(x),
    g=lambda: cos(x),
    xrange=lambda: range_slider(-3, 3, default=(0, 1), label='x-range'),
    yrange=lambda: "auto",
    a=lambda: 1,
    action=lambda: selector(
        [
            "f",
            "df/dx",
            "int f",
            "num f",
            "den f",
            "1/f",
            "finv",
            "f+a",
            "f-a",
            "f*a",
            "f/a",
            "f^a",
            "f(x+a)",
            "f(x*a)",
            "f+g",
            "f-g",
            "f*g",
            "f/g",
            "f(g)",
        ],
        width=15,
        nrows=5,
        label="h = ",
    ),
    do_plot=lambda: ("Draw Plots", True),
)
def function_tool(f, g, xrange, yrange, a, action, do_plot):
    r"""
    `Function Plotting Tool <http://wiki.sagemath.org/interact/calculus#Functiontool>`_
    (by William Stein (?))

    INPUT:

    - ``f`` -- function f(x)
    - ``g`` -- function g(x)
    - ``xrange`` -- range for plotting (x)
    - ``yrange`` -- range for plotting ('auto' is default, otherwise a tuple)
    - ``a`` -- factor ``a``
    - ``action`` -- select given operation on or combination of functions
    - ``do_plot`` -- if ``True``, a plot is drawn

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.calculus.function_tool()  # long time
        ...Interactive function <function function_tool at ...> with 7 widgets
          f: EvalText(value='sin(x)', description='f')
          g: EvalText(value='cos(x)', description='g')
          xrange: IntRangeSlider(value=(0, 1), description='x-range', max=3, min=-3)
          yrange: Text(value='auto', description='yrange')
          a: IntSlider(value=1, description='a', max=3, min=-1)
          action: ToggleButtons(description='h = ', options=('f', 'df/dx', 'int f', 'num f', 'den f', '1/f', 'finv', 'f+a', 'f-a', 'f*a', 'f/a', 'f^a', 'f(x+a)', 'f(x*a)', 'f+g', 'f-g', 'f*g', 'f/g', 'f(g)'), value='f')
          do_plot: Checkbox(value=True, description='Draw Plots')
    """
    x = SR.var('x')
    try:
        f = SR(f)
        g = SR(g)
        a = SR(a)
    except TypeError as msg:
        print(msg[-200:])
        print("Unable to make sense of f,g, or a as symbolic expressions in single variable x.")
        return
    if not (isinstance(xrange, tuple) and len(xrange) == 2):
        xrange = (0, 1)
    h = 0
    lbl = ''
    if action == 'f':
        h = f
        lbl = 'f'
    elif action == 'df/dx':
        h = f.derivative(x)
        lbl = r'\frac{\mathrm{d}f}{\mathrm{d}x}'
    elif action == 'int f':
        h = f.integrate(x)
        lbl = r'\int f \,\mathrm{d}x'
    elif action == 'num f':
        h = f.numerator()
        lbl = r'\text{numer(f)}'
    elif action == 'den f':
        h = f.denominator()
        lbl = r'\text{denom(f)}'
    elif action == '1/f':
        h = 1/f
        lbl = r'\frac{1}{f}'
    elif action == 'finv':
        h = solve(f == SR.var('y'), x)[0].rhs()
        lbl = 'f^{-1}(y)'
    elif action == 'f+a':
        h = f+a
        lbl = 'f + a'
    elif action == 'f-a':
        h = f-a
        lbl = 'f - a'
    elif action == 'f*a':
        h = f*a
        lbl = r'f \times a'
    elif action == 'f/a':
        h = f/a
        lbl = r'\frac{f}{a}'
    elif action == 'f^a':
        h = f**a
        lbl = 'f^a'
    elif action == 'f^a':
        h = f**a
        lbl = 'f^a'
    elif action == 'f(x+a)':
        h = f.subs(x=x+a)
        lbl = 'f(x+a)'
    elif action == 'f(x*a)':
        h = f.subs(x=x*a)
        lbl = 'f(xa)'
    elif action == 'f+g':
        h = f+g
        lbl = 'f + g'
    elif action == 'f-g':
        h = f-g
        lbl = 'f - g'
    elif action == 'f*g':
        h = f*g
        lbl = r'f \times g'
    elif action == 'f/g':
        h = f/g
        lbl = r'\frac{f}{g}'
    elif action == 'f(g)':
        h = f(g)
        lbl = 'f(g)'
    html('<center><font color="red">$f = %s$</font></center>' % latex(f))
    html('<center><font color="green">$g = %s$</font></center>' % latex(g))
    html('<center><font color="blue"><b>$h = %s = %s$</b></font></center>' % (lbl, latex(h)))
    if do_plot:
        P = plot(f, xrange, color='red', thickness=2) + \
            plot(g, xrange, color='green', thickness=2) + \
            plot(h, xrange, color='blue', thickness=2)
        if yrange == 'auto':
            show(P, xmin=xrange[0], xmax=xrange[1])
        else:
            yrange = sage_eval(yrange)
            show(P, xmin=xrange[0], xmax=xrange[1], ymin=yrange[0], ymax=yrange[1])


@library_interact(
    expo=lambda: slider(-10, 10, 0.1, 2),
    c_real=lambda: slider(-2, 2, 0.01, 0.5, label="real part const."),
    c_imag=lambda: slider(-2, 2, 0.01, 0.5, label="imag part const."),
    iterations=lambda: slider(1, 100, 1, 20, label="# iterations"),
    zoom_x=lambda: range_slider(-2, 2, 0.01, (-1.5, 1.5), label="Zoom X"),
    zoom_y=lambda: range_slider(-2, 2, 0.01, (-1.5, 1.5), label="Zoom Y"),
    plot_points=lambda: slider(20, 400, 20, default=150, label="plot points"),
    dpi=lambda: slider(20, 200, 10, default=80, label='dpi'),
)
def julia(expo, c_real, c_imag, iterations, zoom_x, zoom_y, plot_points, dpi):
    r"""
    Julia Fractal, based on
    `Julia by Harald Schilly <http://wiki.sagemath.org/interact/fractal#Julia>`_.

    INPUT:

    - ``exponent`` -- exponent ``e`` in `z^e+c`
    - ``c_real`` -- real part of the constant ``c``
    - ``c_imag`` -- imaginary part of the constant ``c``
    - ``iterations`` -- number of iterations
    - ``zoom_x`` -- range slider for zoom in x direction
    - ``zoom_y`` -- range slider for zoom in y direction
    - ``plot_points`` -- number of points to plot
    - ``dpi`` -- dots-per-inch parameter for the plot

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.fractals.julia()  # long time
        ...Interactive function <function julia at ...> with 8 widgets
          expo: FloatSlider(value=2.0, description='expo', max=10.0, min=-10.0)
          c_real: FloatSlider(value=0.5, description='real part const.', max=2.0, min=-2.0, step=0.01)
          c_imag: FloatSlider(value=0.5, description='imag part const.', max=2.0, min=-2.0, step=0.01)
          iterations: IntSlider(value=20, description='# iterations', min=1)
          zoom_x: FloatRangeSlider(value=(-1.5, 1.5), description='Zoom X', max=2.0, min=-2.0, step=0.01)
          zoom_y: FloatRangeSlider(value=(-1.5, 1.5), description='Zoom Y', max=2.0, min=-2.0, step=0.01)
          plot_points: IntSlider(value=150, description='plot points', max=400, min=20, step=20)
          dpi: IntSlider(value=80, description='dpi', max=200, min=20, step=10)
    """
    z = SR.var('z')
    I = CDF.gen()
    f = symbolic_expression(z**expo + c_real + c_imag*I).function(z)
    ff_j = fast_callable(f, vars=[z], domain=CDF)

    from sage.interacts.library_cython import julia

    html('<h2>Julia Fractal</h2>')
    html(r'Recursive Formula: $z \leftarrow z^{%.2f} + (%.2f+%.2f*\mathbb{I})$' % (expo, c_real, c_imag))
    complex_plot(lambda z: julia(ff_j, z, iterations), zoom_x, zoom_y, plot_points=plot_points, dpi=dpi).show(frame=True, aspect_ratio=1)


@library_interact(
    expo=lambda: slider(-10, 10, 0.1, 2),
    iterations=lambda: slider(1, 100, 1, 20, label="# iterations"),
    zoom_x=lambda: range_slider(-2, 2, 0.01, (-2, 1), label="Zoom X"),
    zoom_y=lambda: range_slider(-2, 2, 0.01, (-1.5, 1.5), label="Zoom Y"),
    plot_points=lambda: slider(20, 400, 20, default=150, label="plot points"),
    dpi=lambda: slider(20, 200, 10, default=80, label='dpi'),
)
def mandelbrot(expo, iterations, zoom_x, zoom_y, plot_points, dpi):
    r"""
    Mandelbrot Fractal, based on
    `Mandelbrot by Harald Schilly <http://wiki.sagemath.org/interact/fractal#Mandelbrot>`_.

    INPUT:

    - ``exponent`` -- exponent ``e`` in `z^e+c`
    - ``iterations`` -- number of iterations
    - ``zoom_x`` -- range slider for zoom in x direction
    - ``zoom_y`` -- range slider for zoom in y direction
    - ``plot_points`` -- number of points to plot
    - ``dpi`` -- dots-per-inch parameter for the plot

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.fractals.mandelbrot()  # long time
        ...Interactive function <function mandelbrot at ...> with 6 widgets
          expo: FloatSlider(value=2.0, description='expo', max=10.0, min=-10.0)
          iterations: IntSlider(value=20, description='# iterations', min=1)
          zoom_x: FloatRangeSlider(value=(-2.0, 1.0), description='Zoom X', max=2.0, min=-2.0, step=0.01)
          zoom_y: FloatRangeSlider(value=(-1.5, 1.5), description='Zoom Y', max=2.0, min=-2.0, step=0.01)
          plot_points: IntSlider(value=150, description='plot points', max=400, min=20, step=20)
          dpi: IntSlider(value=80, description='dpi', max=200, min=20, step=10)
    """
    x, z, c = SR.var('x, z, c')
    f = symbolic_expression(z**expo + c).function(z, c)
    ff_m = fast_callable(f, vars=[z,c], domain=CDF)

    from sage.interacts.library_cython import mandel

    html('<h2>Mandelbrot Fractal</h2>')
    html(r'Recursive Formula: $z \leftarrow z^{%.2f} + c$ for $c \in \mathbb{C}$' % expo)
    complex_plot(lambda z: mandel(ff_m, z, iterations), zoom_x, zoom_y, plot_points=plot_points, dpi=dpi).show(frame=True, aspect_ratio=1)


@library_interact(
    N=lambda: slider(1, 500, 1, label="Number of iterations", default=100),
    rule_number=lambda: slider(0, 255, 1, default=110, label="Rule number"),
    size=lambda: slider(1, 11, step_size=1, default=6, label="size of graphic"),
)
def cellular_automaton(N, rule_number, size):
    r"""
    Yields a matrix showing the evolution of a
    `Wolfram's cellular automaton <http://mathworld.wolfram.com/CellularAutomaton.html>`_.

    `Based on work by Pablo Angulo <http://wiki.sagemath.org/interact/misc#CellularAutomata>`_.

    INPUT:

    - ``N`` -- iterations
    - ``rule_number`` -- rule number (0 to 255)
    - ``size`` -- size of the shown picture

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: interacts.fractals.cellular_automaton()  # long time
        ...Interactive function <function cellular_automaton at ...> with 3 widgets
          N: IntSlider(value=100, description='Number of iterations', max=500, min=1)
          rule_number: IntSlider(value=110, description='Rule number', max=255)
          size: IntSlider(value=6, description='size of graphic', max=11, min=1)
    """
    from sage.rings.integer import Integer
    if not 0 <= rule_number <= 255:
        raise ValueError('Invalid rule number')
    binary_digits = Integer(rule_number).digits(base=2)
    rule = binary_digits + [0]*(8-len(binary_digits))

    html('<h2>Cellular Automaton</h2>' +
         '<div style="white-space: normal;">"A cellular automaton is a collection of "colored" cells \
         on a grid of specified shape that evolves through a number of \
         discrete time steps according to a set of rules based on the \
         states of neighboring cells." &mdash; \
         <a target="_blank" href="http://mathworld.wolfram.com/CellularAutomaton.html">Mathworld,\
         Cellular Automaton</a></div>\
         <div>Rule %s expands to %s</div>' % (rule_number, ''.join(map(str,rule)))
        )

    from sage.interacts.library_cython import cellular
    M = cellular(rule, N)
    plot_M = matrix_plot(M, cmap='binary')
    plot_M.show(figsize=[size,size])


@library_interact(
    interval=lambda: range_slider(1, 4000, 10, default=(1, 1000), label='range'),
    show_factors=lambda: True,
    highlight_primes=lambda: True,
    show_curves=lambda: True,
    n=lambda: slider(1, 200, 1, default=89, label="number $n$"),
    dpi=lambda: slider(10, 300, 10, default=100, label='dpi'),
)
def polar_prime_spiral(interval, show_factors, highlight_primes, show_curves, n, dpi):
    r"""
    Polar Prime Spiral interact, based on work by David Runde.

    For more information about the factors in the spiral,
    `visit John Williamson's website <http://www.dcs.gla.ac.uk/~jhw/spirals/index.html>`_.

    INPUT:

    - ``interval`` -- range slider to specify start and end
    - ``show_factors`` -- if ``True``, show factors
    - ``highlight_primes`` -- if ``True``, prime numbers are highlighted
    - ``show_curves`` -- if ``True``, curves are plotted
    - ``n`` -- number `n`
    - ``dpi`` -- dots per inch resolution for plotting

    EXAMPLES:

    Invoked in the notebook, the following command will produce
    the fully formatted interactive mathlet.  In the command line,
    it will simply return the underlying HTML and Sage code which
    creates the mathlet::

        sage: sage.interacts.algebra.polar_prime_spiral()  # long time
        ...Interactive function <function polar_prime_spiral at ...> with 6 widgets
          interval: IntRangeSlider(value=(1, 1000), description='range', max=4000, min=1, step=10)
          show_factors: Checkbox(value=True, description='show_factors')
          highlight_primes: Checkbox(value=True, description='highlight_primes')
          show_curves: Checkbox(value=True, description='show_curves')
          n: IntSlider(value=89, description='number $n$', max=200, min=1)
          dpi: IntSlider(value=100, description='dpi', max=300, min=10, step=10)
    """
    html('<h2>Polar Prime Spiral</h2> \
          <div style="white-space: normal;">\
          For more information about the factors in the spiral, visit \
          <a href="http://www.dcs.gla.ac.uk/~jhw/spirals/index.html" target="_blank">\
          Number Spirals by John Williamson</a>.</div>'
        )

    start, end = interval
    from math import ceil

    from sage.ext.fast_eval import fast_float
    from sage.plot.colors import hue

    if start < 1 or end <= start:
        print("invalid start or end value")
        return
    if n > end:
        print("WARNING: n is greater than end value")
        return
    if n < start:
        print("n < start value")
        return
    nn = SR.var('nn')
    f1 = fast_float(sqrt(nn)*cos(2*pi*sqrt(nn)), 'nn')
    f2 = fast_float(sqrt(nn)*sin(2*pi*sqrt(nn)), 'nn')
    f = lambda x: (f1(x), f2(x))

    list = []
    list2 = []
    if not show_factors:
        for i in srange(start, end, include_endpoint=True):
            if Integer(i).is_pseudoprime():
                list.append(f(i-start+1))  # primes list
            else:
                list2.append(f(i-start+1))  # composites list
        P = points(list)
        R = points(list2, alpha=.1)  # faded composites
    else:
        for i in srange(start, end, include_endpoint=True):
            # Resize each of the dots depending of the number of factors of each number
            list.append(disk((f(i-start+1)),0.05*pow(2,len(factor(i))-1), (0,2*pi)))
            if Integer(i).is_pseudoprime() and highlight_primes:
                list2.append(f(i-start+1))
        P = Graphics()
        for g in list:
            P += g
        p_size = 5  # the orange dot size of the prime markers
        if not highlight_primes:
            list2 = [(f(n-start+1))]
        R = points(list2, hue=.1, pointsize=p_size)

    if n > 0:
        html('$n = %s$' % factor(n))

        p = 1
        # The X which marks the given n
        W1 = disk((f(n-start+1)), p, (pi/6, 2*pi/6), alpha=.1)
        W2 = disk((f(n-start+1)), p, (4*pi/6, 5*pi/6), alpha=.1)
        W3 = disk((f(n-start+1)), p, (7*pi/6, 8*pi/6), alpha=.1)
        W4 = disk((f(n-start+1)), p, (10*pi/6, 11*pi/6), alpha=.1)
        Q = W1 + W2 + W3 + W4

        n -= start - 1  # offset n for different start values to ensure accurate plotting
        if show_curves:
            begin_curve = 0
            t = SR.var('t')
            a = 1.0
            b = 0.0
            S = int(sqrt(n))
            if n <= S * (S + 1):
                c = n - S**2
            else:
                c = n - (S + 1)**2
            c2 = n - S * (S + 1)
            html('Pink Curve:  $n^2 + %s$' % c)
            html('Green Curve: $n^2 + n + %s$' % c2)
            m = SR.var('m')
            g = symbolic_expression(a*m**2+b*m+c).function(m)
            r = symbolic_expression(sqrt(g(m))).function(m)
            theta = symbolic_expression(r(m) - m*sqrt(a)).function(m)
            S1 = parametric_plot(((r(t))*cos(2*pi*(theta(t))),(r(t))*sin(2*pi*(theta(t)))),
                                 (begin_curve, ceil(sqrt(end-start))),
                                 color=hue(0.8), thickness=.3)  # pink line

            b = 1
            c = c2
            g = symbolic_expression(a*m**2+b*m+c).function(m)
            r = symbolic_expression(sqrt(g(m))).function(m)
            theta = symbolic_expression(r(m) - m*sqrt(a)).function(m)
            S2 = parametric_plot(((r(t))*cos(2*pi*(theta(t))),(r(t))*sin(2*pi*(theta(t)))),
                                 (begin_curve, ceil(sqrt(end-start))),
                                 color=hue(0.6), thickness=.3)  # green line

            show(R+P+S1+S2+Q, aspect_ratio=1, axes=False, dpi=dpi)
        else:
            show(R+P+Q, aspect_ratio=1, axes=False, dpi=dpi)
    else:
        show(R+P, aspect_ratio=1, axes=False, dpi=dpi)
